#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Var', 'VarList', '_VarData']

import logging
import types
import sys
import weakref

from pyutilib.component.core import alias
import pyutilib.math
import pyutilib.misc
from pyutilib.enum import Enum

from numvalue import NumericValue, value, as_numeric, is_fixed
from set_types import BooleanSet, IntegerSet, Reals
from component import Component
from indexed_component import IndexedComponent
from misc import apply_indexed_rule
from misc import create_name
from sets import Set, _SetContainer
from coopr.pyomo.base.util import isfunctor

logger = logging.getLogger('coopr.pyomo')

class _VarData(NumericValue):
    """
    This class defines the data for a single variable.

    Constructor Arguments:
        name        The name of this variable.  Use None for a default value.
        domain      The domain of this variable.  Use None for a default value.
        component   The Var object that owns this data.

    Public Class Attributes:
        active      If True, then this variable is active in the model.
        component   A weakref to the component object that owns this variable index.
        index       The index of this variable within the Var component.
        domain      The domain of this variable.
        fixed       If True, then this variable is treated as a fixed constant in the model.
        initial     The default initial value for this variable.
        lb          A lower bound for this variable.  The lower bound can be either 
                        numeric constants, parameter values, expressions or any object that 
                        can be called with no arguments.
        ub          A upper bound for this variable.  The upper bound can be either 
                        numeric constants, parameter values, expressions or any object that 
                        can be called with no arguments.
        name        A text string that provides an easy-to-read representation of the
                        variable/index pair (e.g. var[x,y,z])
        stale       A Boolean indicating whether the value of this variable is legitimiate -
                    or, in other words, if it should be considered legitimate for purposes 
                    of reporting or other interrogation. 
        value       The numeric value of this variable.
    """

    __slots__ = ['component','index','initial','active','lb','ub','fixed','stale']

    def __init__(self, name, domain, component):
        """
        Constructor
        """
        # The following three lines are equivalent to calling the
        # basic NumericValue constructor, i.e., as follows:
        # NumericValue.__init__(self, name, domain, None, False)
        self.name = name
        self.domain = domain
        self.value = None
        #
        self.active = True
        if component is None:
            self.component = None
        else:
            self.component = weakref.ref(component)
        self.fixed = False
        self.index = None
        self.initial = None
        # IMPT: The type of the lower and upper bound attributes can either be atomic numeric types
        #       in Python, expressions, etc. Basically, they can be anything that passes an "is_fixed" test.
        self.lb = None
        self.ub = None

        self.stale = True

    def __str__(self):
        """
        Return a string representation of the variable.  If the name attribute is None,
        then return ''
        """
        if self.name is None:
            return ""
        else:
            return self.name

    def __getstate__(self):
        """
        This method must be defined because this class uses slots.
        """
        result = NumericValue.__getstate__(self)
        for i in _VarData.__slots__:
            result[i] = getattr(self, i)
        if type(result['component']) is weakref.ref:
            result['component'] = result['component']()
        return result

    def __setstate__(self, state):
        """
        This method must be defined because this class uses slots.
        """
        result = NumericValue.__getstate__(self)
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)
        if self.component is not None:
            self.component = weakref.ref(self.component)

    def get_suffix_value(self, suffix):
        return self.component().get_suffix_value(suffix, self.index)

    def set_suffix_value(self, suffix, value, define=False):
        self.component().set_suffix_value(suffix, value, self.index, define)

    def activate(self):
        self.active=True

    def deactivate(self):
        self.active=False

    def setlb(self, value):
        if value is None:
            self.lb = None
        else:
            if is_fixed(value):
                self.lb = value
            else:
                raise ValueError(
                    "Non-fixed input of type '%s' supplied as variable lower "
                    "bound - legal types must be fixed expressions or variables."
                    % (type(value),) )

    def setub(self, value):
        if value is None:
            self.ub = None
        else:
            if is_fixed(value):
                    self.ub = value
            else:
                raise ValueError(
                    "Non-fixed input of type '%s' supplied as variable upper "
                    "bound - legal types are fixed expressions or variables."
                    "parameters"
                    % (type(value),) )

    def is_fixed(self):
        if self.fixed:
            return True
        return False

    def is_constant(self):
        return False

    def polynomial_degree(self):
        if self.fixed:
            return 0
        return 1

    def is_binary(self):
        return self.component().is_binary(self.index)

    def is_integer(self):
        return self.component().is_integer(self.index)

    def is_continuous(self):
        return self.component().is_continuous(self.index)

    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, str(self),

#
# Variable attributes:
#
# Single variable:
#
# x = Var()
# x.fixed = True
# x.domain = Reals
# x.name = "x"
# x.setlb(-1.0)
# x.setub(1.0)
# x.initial = 0.0
# x = -0.4
#
# Array of variables:
#
# y = Var(set1,set2,...,setn)
# y.fixed = True (fixes all variables)
# y.domain = Reals
# y.name = "y"
# y[i,j,...,k] (returns value of a given index)
#
# y[i,j,...,k].fixed
# y[i,j,...,k].lb()
# y[i,j,...,k].ub()
# y[i,j,...,k].initial
#

class Var(IndexedComponent):
    """A numeric variable, which may be defined over a index"""

    """
    Constructor Arguments:
        name        The name of this variable
        index       The index set that defines the distinct variables.
                        By default, this is None, indicating that there
                        is a single variable.
        domain      A set that defines the type of values that
                        each variable must be.
        default     A set that defines default values for this
                        variable.
        bounds      A rule for defining bounds values for this
                        variable.
        rule        A rule for setting up this variable with
                        existing model data
    """

    alias("Var", doc="Decision variables in a model.", subclass=True)

    def __new__(cls, *args, **kwds):
        if cls != Var:
            return super(Var, cls).__new__(cls)
        if args == ():
            return _VarElement.__new__(_VarElement)
        else:
            return _VarArray.__new__(_VarArray)

    def __init__(self, *args, **kwd):
        # Default keyword values
        self._initialize = kwd.pop('initialize', None )
        self.domain = kwd.pop('within', Reals )
        self.domain = kwd.pop('domain', self.domain )
        self.bounds = kwd.pop('bounds', None )

        self._defer_domain = False

        #Handle within=not_pyomo_object 
        if not getattr(self.domain, 'virtual', False):
            if self.domain == [] or self.domain == set([]):
                raise ValueError,"Attempting to set a variable's domain to an empty set"
            else: 
                from rangeset import RangeSet
                if type(self.domain) is RangeSet:
                    self.bounds = (self.domain._start,self.domain._end)
                    domain_name = self.domain.name
                    self.domain = IntegerSet()
                    self.domain.name = domain_name
                elif type(self.domain) is _SetContainer:
                    if self.domain.initialize is None:
                        self._defer_domain = True
                    elif self.domain.initialize == []:
                        raise ValueError,"Attempting to set a variable's domain to an empty Set"
                    else:
                        self._defer_domain = False
                        domain_name = self.domain.name
                        self.domain = self.domain.initialize
                        self.domain.sort()
                        if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                            raise ValueError,"Attempting to set a variable's domain to an improperly formatted Set " \
                                           + "-- elements are missing or duplicates exist: {0}".format(self.domain)
                        elif not all(type(elt) is int for elt in self.domain):
                            raise ValueError,"Attempting to set a veriable's domain to a Set with noninteger elements: {0}".format(self.domain)
                        else:
                            self.bounds = (self.domain[0],self.domain[-1])
                            self.domain = IntegerSet()
                            self.domain.name = domain_name
                elif type(self.domain) is list:
                    self.domain.sort()
                    if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                        raise ValueError,"Attempting to set a variable's domain to an improperly formatted set " \
                                       + "-- elements are missing or duplicates exist: {0}".format(self.domain)
                    elif not all(type(elt) is int for elt in self.domain):
                        raise ValueError,"Attempting to set a veriable's domain to a set with noninteger elements: {0}".format(self.domain)
                    else:
                        self.bounds = (self.domain[0],self.domain[-1])
                        domain_name = "[{0}..{1}]".format(self.domain[0],self.domain[-1])
                        self.domain = IntegerSet()
                        self.domain.name = domain_name
                elif type(self.domain) is set:
                    self.domain = list(self.domain) 
                    self.domain.sort()
                    if len(self.domain) != self.domain[-1] - self.domain[0] + 1:
                        raise ValueError,"Attempting to set a variable's domain to an improperly formatted set " \
                                       + "-- elements are missing or duplicates exist: {0}".format(self.domain)
                    elif not all(type(elt) is int for elt in self.domain):
                        raise ValueError,"Attempting to set a veriable's domain to a set with noninteger elements: {0}".format(self.domain)
                    else:
                        self.bounds = (self.domain[0],self.domain[-1])
                        domain_name = "[{0}..{1}]".format(self.domain[0],self.domain[-1])
                        self.domain = IntegerSet()
                        self.domain.name = domain_name
                elif type(self.domain) is xrange:
                    self.bounds = (self.domain[0],self.domain[-1])
                    domain_name = "[{0}..{1}]".format(self.bounds[0],self.bounds[-1])
                    self.domain = IntegerSet()
                    self.domain.name = domain_name

        kwd.setdefault('ctype', Var)

        IndexedComponent.__init__(self, *args, **kwd)

        # check to see if domains for individual _VarData objects are specified via a rule.
        if isfunctor(self.domain):
            self._domain_rule = self.domain
            self.domain = None # there is no general domain for the collection of variables.
        else:
            self._domain_rule = None # there is no domain rule!

        # cached for efficiency purposes - isinstance is not cheap. 
        # only truly valid if there is no domain rule specified.
        if self.domain is not None:
            self._is_binary = isinstance(self.domain, BooleanSet)
            # second case is required in the case of deferred domains (the set may not be constructed yet, so hasn't been translated to an IntegerSet)
            self._is_integer = isinstance(self.domain, IntegerSet) or (type(self.domain) is _SetContainer)
            self._is_continuous = not (self._is_binary or self._is_integer)
        else:
            self._is_binary = None
            self._is_integer = None
            self._is_continuous = None

        # the following are only applicable if a domain rule *has* been specified.
        self._binary_keys = []
        self._integer_keys = []
        self._continuous_keys = []

    def as_numeric(self):
        if None in self._data:
            return self._data[None]
        return self

    def is_indexed(self):
        return self._ndim > 0

    def is_expression(self):
        return False

    def is_relational(self):
        return False

    def keys(self):
        return self._data.keys()

    # sets the "stale" attribute of every var index to True.
    def flag_as_stale(self):
        for index, var_data in self._data.iteritems():
            var_data.stale = True

    # returns a dictionary of index-value pairs.
    def extract_values(self):
        return dict(((index, varval.value) for index, varval in self._data.iteritems()))

    # takes as input a dictionary of index-value pairs.
    def store_values(self, new_values):
        for index, varval in self._data.iteritems():
            varval.value = new_values[index]
            varval.stale = False

    def is_binary(self, index):
        if self._domain_rule is None:
            return self._is_binary
        else:
            return index in self._binary_keys

    def is_integer(self, index):
        if self._domain_rule is None:
            return self._is_integer
        else:
            return index in self._integer_keys

    def is_continuous(self, index):
        if self._domain_rule is None:
            return self._is_continuous
        else:
            return index in self._continuous_keys

    def binary_keys(self):
        """ Returns the keys of all binary variables """
        if self._domain_rule is None:
            if self._is_binary:
                return self._data.keys()
            else:
                return []
        else:
            return self._binary_keys

    def integer_keys(self):
        """ Returns the keys of all integer variables """
        if self._domain_rule is None:
            if self._is_integer:
                return self._data.keys()
            else:
                return []
        else:
            return self._integer_keys

    def continuous_keys(self):
        """ Returns the keys of all continuous variables """
        if self._domain_rule is None:
            if self._is_continuous:
                return self._data.keys()
            else:
                return []
        else:
            return self._continuous_keys

    def __iter__(self):
        return self._data.iterkeys()

    def iterkeys(self):
        return self._data.iterkeys()

    def itervalues(self):
        return self._data.itervalues()

    def iteritems(self):
        return self._data.iteritems()

    def __contains__(self, ndx):
        return ndx in self._data

    def dim(self):
        return self._ndim

    def reset(self):
        for value in self._data.itervalues():
            value.set_value(value.initial)

    def __len__(self):
        return len(self._data)

    def __setitem__(self,ndx,val):
        #print "HERE",ndx,val, self._valid_value(val,False), self.domain # XXX debugging
        if (len(self._index) == 1) and (None in self._index) and (ndx is not None): # a somewhat gory way to see if you have a singleton - the len()=1 check is needed to avoid "None" being passed illegally into set membership validation rules.
            # allow for None indexing if this is truly a singleton
            msg = "Cannot set an array value in singleton variable '%s'"
            raise KeyError, msg % self.name

        if ndx not in self._index:
            msg = "Cannot set the value of array variable '%s' with invalid " \
                  "index '%s'"
            raise KeyError, msg % ( self.name, str(ndx) )

        if not self._valid_value(val,False):
            msg = "Cannot set variable '%s[%s]' with invalid value: %s"
            raise ValueError, msg % ( self.name, str(ndx), str(val) )
        self._data[ndx].value = val
        self._data[ndx].stale = False

    def __getitem__(self,ndx):
        """This method returns a _VarData object.  This object can be
           coerced to a numeric value using the value() function, or using
           explicity coercion with float().
        """
        try:
            return self._data[ndx]
        except KeyError: # thrown if the supplied index is hashable, but not defined.
            msg = "Unknown index '%s' in variable %s;" % (str(ndx), self.name)
            if (isinstance(ndx, (tuple, list)) and len(ndx) != self.dim()):
                msg += "    Expecting %i-dimensional indices" % self.dim()
            else:
                msg += "    Make sure the correct index sets were used.\n"
                msg += "    Is the ordering of the indices correct?"
            raise KeyError, msg
        except TypeError, msg: # thrown if the supplied index is not hashable
            msg2 = "Unable to index variable %s using supplied index with " % self.name
            msg2 += str(msg)
            raise TypeError, msg2

    def _add_indexed_member(self, ndx):
        # Check for domain rules
        if self._domain_rule is not None:
            domain = apply_indexed_rule( self, self._domain_rule,
                                         self._parent(), ndx )
            if isinstance(domain, BooleanSet):
                self._binary_keys.append(ndx)
            elif isinstance(domain, IntegerSet):
                self._integer_keys.append(ndx)
            else:
                self._continuous_keys.append(ndx)
        else:
            domain = self.domain

        new_data = _VarData(create_name(self.name,ndx), domain, self)
        new_data.index = ndx
        
        self._data[ndx] = new_data


    def construct(self, data=None):
        if __debug__:   #pragma:nocover
            if logger.isEnabledFor(logging.DEBUG):
                logger.debug("Constructing Variable, name=%s, from data=%s", self.name, `data`)
        if self._constructed:
            return
        self._constructed=True
        #
        # Construct _VarData objects for all index values
        #
        if self._ndim > 0:
            
            if type(self._initialize) is dict:
                self._index = self._initialize.keys()

            for ndx in self._index:
                # Ignore a None index.  This occurs when an empty VarList is
                # created;  the empty implicit set has the None index.
                if ndx is not None: 
                    self._add_indexed_member(ndx)


        #
        # Define the _XXX_keys objects if domain isn't a rule;
        # they were defined individually above in the case of
        # a rule
        #
        if self._domain_rule is None:
            if isinstance(self.domain, BooleanSet):
                self._binary_keys = self._data.keys()
            elif isinstance(self.domain, IntegerSet):
                self._integer_keys = self._data.keys()
            else:
                self._continuous_keys = self._data.keys()

        #
        # Initialize values with a dictionary if provided
        #
        if self._initialize is not None:
            #
            # Initialize values with the _rule function if provided
            #
            if self._initialize.__class__ is types.FunctionType:
                for key in self._data:
                    if key is None:
                        val = self._initialize(self._parent())
                    else:
                        val = apply_indexed_rule( self, self._initialize,
                                                  self._parent(), key )
                    val = value(val)
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False
            elif self._initialize.__class__ is dict:
                for key in self._initialize:
                    val = self._initialize[key]
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False                    
            else:
                for key in self._index:
                    val = self._initialize
                    self._valid_value(val, True)
                    self._data[key].value = self._data[key].initial = val
                    self._data[key].stale = False                    
        #
        # Initialize bounds with the bounds function if provided
        #
        if self.bounds is not None:

            if type(self.bounds) is tuple:

                # bounds are specified via a tuple - same lower and upper bounds for all var values!

                (lb, ub) = self.bounds

                # do some simple validation that the bounds are actually finite - otherwise, set them to None.
                if (lb is not None) and (not pyutilib.math.is_finite(value(lb))):
                    lb = None
                if (ub is not None) and (not pyutilib.math.is_finite(value(ub))):
                    ub = None

                for key,varval in self._data.iteritems():
                    if lb is not None:
                        varval.setlb(lb)
                    if ub is not None:
                        varval.setub(ub)

            else:

                # bounds are specified via a function

                for index, varval in self._data.iteritems():
                    if index is None:
                        (lb, ub) = self.bounds(self._parent())
                    else:
                        (lb, ub) = apply_indexed_rule( self, self.bounds,
                                                       self._parent(), index )
                    varval.setlb(lb)
                    varval.setub(ub)

                for varval in self._data.itervalues():
                    if varval.lb is not None and not pyutilib.math.is_finite(value(varval.lb)):
                        varval.setlb(None)
                    if varval.ub is not None and not pyutilib.math.is_finite(value(varval.ub)):
                        varval.setub(None)
        #
        # Iterate through all variables, and tighten the bounds based on
        # the domain bounds information.
        #
        # Only done if self.domain is not a rule. If it is, _VarArray level
        # bounds become meaningless, since the individual _VarElement objects
        # likely have more restricted domains.
        #
        if self._domain_rule is None:
            dbounds = self.domain.bounds()
            if not dbounds is None and dbounds != (None,None):
                for varval in self._data.itervalues():
                    if not dbounds[0] is None:
                        if varval.lb is None or                     \
                           dbounds[0] > value(varval.lb):
                            varval.setlb(dbounds[0])

                    if not dbounds[1] is None:
                        if varval.ub is None or                     \
                            dbounds[1] < value(varval.ub):
                            varval.setub(dbounds[1])
        #
        # Setup declared attributes for all variables
        #
        for suffix in self._suffix_declaration:
            self._suffix_value[suffix] = {}

    def fix_all(self):
        for key in self:
            self[key].fixed = True

    def unfix_all(self):
        for key in self:
            self[key].fixed = False


    def pprint(self, ostream=None, verbose=False):
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, "  ",self.name,":",
        if not self.doc is None:
            print >>ostream, self.doc
            print >>ostream, "  ",
        print >>ostream, "\tSize="+str(len(self)),
        if self.domain is not None:
            print >>ostream, "\tDomain="+self.domain.name
        else:
            print >>ostream, "\tDomain=None"
        if self._index_set is not None:
            print >>ostream, "\tIndicies: ",
            for idx in self._index_set:
                print >>ostream, str(idx.name)+", ",
            print ""
        if None in self._data:
            print >>ostream, "\tInitial Value : Lower Bound : Upper Bound : "  \
                             "Current Value: Fixed: Stale"
            lb_value = None
            if self._data[None].lb is not None:
                lb_value = value(self._data[None].lb)
            ub_value = None
            if self._data[None].ub is not None:
                ub_value = value(self._data[None].ub)

            print >>ostream, "\t %s : %s : %s : %s : %s : %s" % (
              str(self._data[None].initial),
              str(lb_value),
              str(ub_value),
              str(self._data[None].value),
              str(self._data[None].fixed),
              str(self._data[None].stale)
            )
        else:
            print >>ostream, "\tKey : Initial Value : Lower Bound : "          \
                             "Upper Bound : Current Value: Fixed: Stale"
            tmp=self._data.keys()
            tmp.sort()
            for key in tmp:
                initial_val = self._data[key].initial
                lb_value = None
                if self._data[key].lb is not None:
                    lb_value = value(self._data[key].lb)
                ub_value = None
                if self._data[key].ub is not None:
                    ub_value = value(self._data[key].ub)

                print >>ostream, "\t%s : %s : %s : %s : %s : %s : %s" % (
                  str(key),
                  str(initial_val),
                  str(lb_value),
                  str(ub_value),
                  str(value(self._data[key].value)),
                  str(value(self._data[key].fixed)),
                  str(self._data[key].stale)
                )


    def display(self, prefix="", ostream=None):
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, prefix+"Variable "+self.name,":",
        print >>ostream, "  Size="+str(len(self)),
        print >>ostream, "Domain="+self.domain.name
        if None in self._data:
            print >>ostream, "%s  Value=%s" % (
              prefix,
              pyutilib.misc.format_io(self._data[None].value)
            )
        else:
            for key in self._data:
                val = self._data[key].value
                print >>ostream, prefix+"  "+str(key)+" : "+str(val)

# a _VarElement is the implementation representing a "singleton" or non-indexed variable.
# NOTE: this class derives from both a "slot"ized base class (_VarData) and a normal
#       class with a dictionary (Var) - beware (although I believe the __getstate__
#       implementation should be attribute-independent).

class _VarElement(Var, _VarData):

    def __init__(self, *args, **kwd):
        _VarData.__init__(self,
                            kwd.get('name', None), 
                            kwd.get('within', kwd.get('domain', Reals)), 
                            self)
        Var.__init__(self, *args, **kwd)
        self._data[None] = self
        self._data[None].index = None

    def __call__(self, exception=True):
        if None in self._data:
            return self._data[None].value
        return None

    def __getstate__(self):
        #
        # picks up all slots in the _VarData class, and in particular
        # makes sure the "var" and "component" values are not weakrefs,
        # but the actual objects themselves.
        #
        varvalue_result = _VarData.__getstate__(self)
        varvalue_result['component'] = self
        varbase_result = Var.__getstate__(self)
        return dict(varvalue_result.items() + varbase_result.items())

    def __setstate__(self, state):
        for (slot_name, value) in state.iteritems():
            setattr(self, slot_name, value)

        self._data[None].component = weakref.ref(self)
        # this is hackish, in that it replicates code from the base Component setstate method - revisit at some point.
        if '_parent' in self.__dict__.keys() and self._parent is not None and type(self._parent) != weakref.ref:
            self._parent = weakref.ref(self._parent)

    def is_constant(self):
        return _VarData.is_constant(self)

# a _VarArray is the implementation representing an indexed variable.

class _VarArray(Var):
    
    def __init__(self, *args, **kwds):
        Var.__init__(self, *args, **kwds)
        self._dummy_val = _VarData(kwds.get('name', None), 
                                    kwds.get('within', kwds.get('domain', Reals)), 
                                    self)

    def __float__(self):
        raise TypeError, "Cannot access the value of array variable "+self.name

    def __int__(self):
        raise TypeError, "Cannot access the value of array variable "+self.name

    def _valid_value(self,value,use_exception=True):
        return self._dummy_val._valid_value(value, use_exception)

    def set_value(self, value):
        msg = "Cannot specify the value of array variable '%s'"
        raise ValueError, msg % self.name

    def __str__(self):
        return self.name

class VarList(_VarArray):
    """
    Variable-length indexed variable objects used to construct Pyomo models.
    """

    alias("VarList", "Variable-length list of decision variables in a model.")

    End = ( 1003, )

    def __init__(self, *args, **kwargs):
        if len(args) > 0:
            raise ValueError, "Cannot specify indices for a VarList object"
        # Construct parents...
        self._hidden_index = Set()
        _VarArray.__init__(self, self._hidden_index, **kwargs)
        self._ndim = 1
        self._nvars = 0

    def is_fixed(self):
        if self._nvars == 0:
            return False
        for idx in xrange(self._nvars):
            if not self[idx].is_fixed():
                return False
        return True

    def is_constant(self):
        return False

    def polynomial_degree(self):
        if self.is_fixed():
            return 0
        return 1

    def construct(self, data=None):
        if __debug__:
            logger.debug("Constructing variable list %s",self.name)
        self._hidden_index.construct()
        _VarArray.construct(self, data)

    def add(self):
        self._hidden_index.add(self._nvars)
        self._add_indexed_member(self._nvars)
        self._nvars += 1
        return self[self._nvars-1]






#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the FAST README.txt file.
#  _________________________________________________________________________

import sys
import logging
import itertools
from coopr.pyomo.base import Constraint, ConstraintList, Objective
from coopr.pyomo.base import IPyomoPresolver, IPyomoPresolveAction
import coopr.pyomo.expr

from coopr.pyomo.expr import generate_canonical_repn

import coopr.pyomo.base.connector 

def preprocess_objectives(model, var_id_map):

    if getattr(model,'skip_canonical_repn',False):
        return model

    #
    # TBD: This assumes the objective(s) are always in the main model
    #
    active_objectives = model.active_components(Objective)
    for key, obj in active_objectives.iteritems():
        # number of objective indicies with non-trivial expressions
        num_nontrivial = 0

        for ondx, objective_data in obj._data.iteritems():
            if not objective_data.active:
                continue
            if objective_data.expr is None:
                raise ValueError, "No expression has been defined for objective %s" % str(key)

            try:
                objective_data.repn = generate_canonical_repn(objective_data.expr, var_id_map)
            except Exception, err:
                logging.getLogger('coopr.pyomo').error\
                    ( "exception generating a canonical representation for objective %s (index %s): %s" \
                          % (str(key), str(ondx), str(err)) )
                raise
            if not coopr.pyomo.expr.canonical_is_constant(objective_data.repn):
                num_nontrivial += 1

        if num_nontrivial == 0:
            obj.trivial = True
        else:
            obj.trivial = False

def preprocess_constraint(model, constraint, var_id_map):

    # number of constraint indicies with non-trivial bodies
    num_nontrivial = 0

    for index, constraint_data in constraint._data.iteritems():

        if not constraint_data.active:
            continue

        if constraint_data.lin_body is not None:
            # if we already have the linear encoding of the constraint body, skip canonical expression
            # generation. but we still need to assess constraint triviality.
            if not coopr.pyomo.expr.is_linear_expression_constant(constraint_data.lin_body):
                num_nontrivial += 1
            continue

        if constraint_data.body is None:
            raise ValueError, "No expression has been defined for the body of constraint %s, index=%s" % (str(constraint.name), str(index))

        # FIXME: This is a huge hack to keep canonical_repn from trying to generate representations
        #        representations of Constraints with Connectors (which will be deactivated once they
        #        have been expanded anyways). This can go away when preprocess is moved out of the
        #        model.create() phase and into the future model validation phase. (ZBF)
        ignore_connector = False
        if hasattr(constraint_data.body,"_args") and constraint_data.body._args is not None:
            for arg in constraint_data.body._args:
                if arg.__class__ is coopr.pyomo.base.connector._ConnectorElement:
                    ignore_connector = True
        if ignore_connector:
            #print "Ignoring",constraint.name,index
            continue

        try:
            constraint_data.repn = generate_canonical_repn(constraint_data.body, var_id_map)
        except Exception:
            logging.getLogger('coopr.pyomo').error \
                                                    ( "exception generating a canonical representation for constraint %s (index %s)" \
                                                      % (str(constraint.name), str(index)) )
            raise
        if not coopr.pyomo.expr.canonical_is_constant(constraint_data.repn):
            num_nontrivial += 1

    if num_nontrivial == 0:
        constraint.trivial = True
    else:
        constraint.trivial = False

def preprocess_constraints(model, var_id_map):

    if getattr(model,'skip_canonical_repn',False):
        return model

    #
    # FIXME: This is a hack to get block recursion working!
    #

    for block in model.all_blocks():
        active_constraints = block.active_components(Constraint)
        active_constraintlists = block.active_components(ConstraintList)
        for constraint_name, constraint in itertools.chain(active_constraints.iteritems(), active_constraintlists.iteritems()):
            preprocess_constraint(model, constraint, var_id_map)


@coopr.core.coopr_api(namespace='pyomo.model')
def compute_canonical_repn(data, model=None):
    """
    This plugin computes the canonical representation for all
    objectives and constraints linear terms.  All modifications are
    in-place, i.e., components of the input instance are modified.

    NOTE: The idea of this module should be generaized. there are
    two key functionalities: computing a version of the expression
    tree in preparation for output and identification of trivial
    constraints.

    NOTE: this leaves the trivial constraints in the model.

    We break out preprocessing of the objectives and constraints
    in order to avoid redundant and unnecessary work, specifically
    in contexts where a model is iteratively solved and modified.
    we don't have finer-grained resolution, but we could easily
    pass in a Constraint and an Objective if warranted.

    Required:
        model:      A concrete model instance.
    """
    var_id_map = {}
    preprocess_constraints(model, var_id_map)
    preprocess_objectives(model, var_id_map)




from coopr.pyomo.check.checker import IterativeTreeChecker

import ast

class Imports(IterativeTreeChecker):
    """
    Check that an import for the coopr.pyomo package exists
    somewhere within the initial imports block
    """

    def beginChecking(self, runner, script):
        self.pyomoImported = False

    def endChecking(self, runner, script):
        if not self.pyomoImported:
            self.problem("The model script never imports coopr.pyomo.")

    def checkerDoc(self):
        return """\
        You may have trouble creating model components.
        Consider adding the following statement at the
        top of your model file:
            from coopr.pyomo import *
        """

    def check(self, runner, script, info):
        if isinstance(info, ast.Import):
            for name in info.names:
                if isinstance(name, ast.alias):
                    if name.name == 'coopr.pyomo':
                        self.pyomoImported = True

        if isinstance(info, ast.ImportFrom):
            if info.module == 'coopr.pyomo':
                self.pyomoImported = True



from coopr.pyomo.check.hooks import ModelTrackerHook

from _rulebase import _ModelRuleChecker

import ast

class ModelValue(_ModelRuleChecker):
    ModelTrackerHook()
    
    def checkerDoc(self):
        return """\
        Comparisons done on model objects should generally be wrapped in
        a call to value(). The comparison alone will not produce a True/False
        result, but instead generate an expression for later use in a model.
        """

    def check(self, runner, script, info):
        # call superclass to execute checkBody() as necessary
        _ModelRuleChecker.check(self, runner, script, info)

        # also check global If statements
        if isinstance(info, ast.If):
            self.checkCompare(info.test, script = script)

    def checkBody(self, funcdef):
        """Check the body of a function definition for model comparisons local
           to its scope (i.e. using its model argument)."""

        if not isinstance(funcdef.args.args[0], ast.Name):
            return
        modelArg = funcdef.args.args[0].id

        for bodyNode in funcdef.body:
            for node in ast.walk(bodyNode):
                if isinstance(node, ast.If):
                    self.checkCompare(node.test, modelName = modelArg)

    def checkCompare(self, compare, modelName = None, script = None):
        """Check an AST Compare node - iterate for Attribute nodes and match
           against modelName argument. Recurse for script's model defs."""

        if modelName is None and script is None:
            return
        
        if modelName is not None:
            valueCallArgs = []
            generatorExps = []
            for node in ast.walk(compare):
                if isinstance(node, ast.Attribute):
                    if isinstance(node.value, ast.Name):
                        if node.value.id == modelName:
                            wrapped = self.checkWrapped(node, valueCallArgs, generatorExps)
                            if not wrapped:
                                self.problem("Comparison on attribute {0}.{1} not wrapped in value()".format(modelName, node.attr), lineno=compare.lineno)
                elif isinstance(node, ast.Call):
                    if isinstance(node.func, ast.Name):
                        if node.func.id == 'value':
                            valueCallArgs.append(node.args)
                elif isinstance(node, ast.GeneratorExp):
                    generatorExps.append(node)

        if script is not None:
            for name in script.modelVars:
                self.checkCompare(compare, modelName = name)

    def checkWrapped(self, attrNode, valueCallArgs, generatorExps):
        """check if the given attribute node has been 'wrapped', either
           in a value() call or as part of the iterator in a generator
           expression"""
        for i in range(len(valueCallArgs)):
            for j in range(len(valueCallArgs[i])):
                # i = call idx (to return), j = arg idx
                argNode = valueCallArgs[i][j]
                for subnode in ast.walk(argNode):
                    if subnode is attrNode:
                        return True
        for genExp in generatorExps:
            for generator in genExp.generators:
                for subnode in ast.walk(generator.iter):
                    if subnode is attrNode:
                        return True
        return False



#
# Unit Tests for Elements of a Model
#
# Test             Class to test the Model class
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo.base import IntegerSet
from coopr.pyomo import *
from coopr.opt import *
from coopr.pyomo.base.var import _VarElement
import pyutilib.th as unittest
import pyutilib.services

class PyomoModel(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class Test(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        if os.path.exists("unknown.lp"):
            os.unlink("unknown.lp")
        pyutilib.services.TempfileManager.clear_tempfiles()

    def test_clear_attribute(self):
        """ Coverage of the _clear_attribute method """
        self.model.A = Set()
        self.assertEqual(self.model.A.name,"A")
        self.model.A = Var()
        self.assertEqual(self.model.A.name,"A")
        self.model.A = Param()
        self.assertEqual(self.model.A.name,"A")
        self.model.A = Objective()
        self.assertEqual(self.model.A.name,"A")
        self.model.A = Constraint()
        self.assertEqual(self.model.A.name,"A")
        self.model.A = Set()
        self.assertEqual(self.model.A.name,"A")

    def test_set_attr(self):
        self.model.x = Param()
        self.model.x = None

    def test_write(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.x)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.write()

    def test_write2(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.x)
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
            return (1, model.x[1]+model.x[2], 2)
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        self.instance.write()

    def test_write3(self):
        """Test that the summation works correctly, even though param 'w' has a default value"""
        self.model.J = RangeSet(1,4)
        self.model.w=Param(self.model.J, default=4)
        self.model.x=Var(self.model.J)
        def obj_rule(instance):
            return summation(instance.w, instance.x)
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.obj[None].expr._args), 4)

    def test_solve1(self):
        if not pyutilib.services.registered_executable("glpsol"):
            return
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.x)
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
            expr = 0
            for i in model.A:
                expr += i*model.x[i]
            return expr == 0
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        opt = SolverFactory('glpk')
        results = opt.solve(self.instance, keepFiles=True)
        results.write(filename=currdir+"solve1.out", format='json')
        self.assertMatchesJsonBaseline(currdir+"solve1.out",currdir+"solve1.txt", tolerance=1e-4)
        #
        def d_rule(model):
            return model.x[1] > 0
        self.model.d = Constraint(rule=d_rule)
        self.model.d.deactivate()
        self.instance = self.model.create()
        results = opt.solve(self.instance, keepFiles=True)
        results.write(filename=currdir+"solve1x.out", format='json')
        self.assertMatchesJsonBaseline(currdir+"solve1x.out",currdir+"solve1.txt", tolerance=1e-4)
        #
        self.model.d.activate()
        self.instance = self.model.create()
        results = opt.solve(self.instance, keepFiles=True)
        results.write(filename=currdir+"solve1a.out", format='json')
        self.assertMatchesJsonBaseline(currdir+"solve1a.out",currdir+"solve1a.txt", tolerance=1e-4)
        #
        self.model.d.deactivate()
        def e_rule(model, i):
            return model.x[i] > 0
        self.model.e = Constraint(self.model.A, rule=e_rule)
        self.instance = self.model.create()
        for i in self.instance.A:
            self.instance.e[i].deactivate()
        results = opt.solve(self.instance, keepFiles=True)
        results.write(filename=currdir+"solve1b.out", format='json')
        self.assertMatchesJsonBaseline(currdir+"solve1b.out",currdir+"solve1b.txt", tolerance=1e-4)

    def Xtest_load1(self):
        """Testing loading of vector solutions"""
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.x)
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
            expr = 0
            for i in model.A:
                expr += i*model.x[i]
            return expr == 0
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        ans = [0.75]*4
        self.instance.load(ans)
        self.instance.display(currdir+"solve1.out")
        self.assertFileEqualsBaseline(currdir+"solve1.out",currdir+"solve1c.txt")

    def Xtest_solve2(self):
        """
        WEH - this is disabled because glpk appears to work fine
        on this example.  I'm not quite sure what has changed that has
        impacted this test...
        """
        if not pyutilib.services.registered_executable("glpsol"):
            return
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            expr = 0
            for i in model.A:
                expr += model.x[i]
            return expr
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        opt = solvers.GLPK(keepFiles=True)
        solutions = opt.solve(self.instance)
        solutions.write()
        sys.exit(1)
        try:
            self.instance.load(solutions)
            self.fail("Cannot load a solution with a bad solver status")
        except ValueError:
            pass

    def test_solve3(self):
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            expr = 0
            for i in model.A:
                expr += model.x[i]
            return expr
        self.model.obj = Objective(rule=obj_rule)
        self.instance = self.model.create()
        self.instance.display(currdir+"solve3.out")
        self.assertFileEqualsBaseline(currdir+"solve3.out",currdir+"solve3.txt")

    def test_solve4(self):
        if not pyutilib.services.registered_executable("glpsol"):
            return
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.x)
        self.model.obj = Objective(rule=obj_rule)
        def c_rule(model):
            expr = 0
            for i in model.A:
                expr += i*model.x[i]
            return expr == 0
        self.model.c = Constraint(rule=c_rule)
        self.instance = self.model.create()
        #self.instance.pprint()
        opt = SolverFactory('glpk', keepFiles=True)
        results = opt.solve(self.instance)
        results.write(filename=currdir+'solve4.out', format='json')
        self.assertMatchesJsonBaseline(currdir+"solve4.out",currdir+"solve1.txt", tolerance=1e-4)

    def Xtest_solve5(self):
        """ A draft test for the option to select an objective """
        if not pyutilib.services.registered_executable("glpsol"):
            return
        self.model.A = RangeSet(1,4)
        self.model.x = Var(self.model.A, bounds=(-1,1))
        def obj1_rule(model):
            expr = 0
            for i in model.A:
                expr += model.x[i]
            return expr
        self.model.obj1 = Objective(rule=obj1_rule)
        def obj2_rule(model):
            expr = 0
            tmp=-1
            for i in model.A:
                expr += tmp*i*model.x[i]
                tmp *= -1
            return expr
        self.model.obj2 = Objective(rule=obj2_rule)
        self.instance = self.model.create()
        opt = SolverFactory('glpk', keepfiles=True)
        results = opt.solve(self.instance, objective='obj2')
        results.write(filename=currdir+"solve5.out", format='json')
        self.assertMatchesJsonBaseline(currdir+"solve5.out",currdir+"solve5a.txt", tolerance=1e-4)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________


def to_standard_form(self):
    """
    Produces a standard-form representation of the model. Returns
    the coefficient matrix (A), the cost vector (c), and the
    constraint vector (b), where the 'standard form' problem is

    min/max c'x
    s.t.    Ax = b
            x >= 0

    All three returned values are instances of the array.array
    class, and store Python floats (C doubles).
    """

    from coopr.pyomo.expr import generate_canonical_repn


    # We first need to create an map of all variables to their column
    # number
    colID = {}
    ID2name = {}
    id = 0
    tmp = self.variables().keys()
    tmp.sort()

    for v in tmp:
        colID[v] = id
        ID2name[id] = v
        id += 1

    # First we go through the constraints and introduce slack and excess
    # variables to eliminate inequality constraints
    #
    # N.B. Structure heirarchy:
    #
    # active_components: {class: {attr_name: object}}
    # object -> Constraint: ._data: {ndx: _ConstraintData}
    # _ConstraintData: .lower, .body, .upper
    #
    # So, altogether, we access a lower bound via
    #
    # model.active_components()[Constraint]['con_name']['index'].lower
    #
    # {le,ge,eq}Constraints are
    # {constraint_name: {index: {variable_or_none: coefficient}} objects
    # that represent each constraint. None in the innermost dictionary
    # represents the constant term.
    #
    # i.e.
    #
    # min  x1 + 2*x2 +          x4
    # s.t. x1                         = 1
    #           x2   + 3*x3          <= -1
    #      x1 +                 x4   >= 3
    #      x1 + 2*x2 +      +   3*x4 >= 0
    #
    #
    # would be represented as (modulo the names of the variables,
    # constraints, and indices)
    #
    # eqConstraints = {'c1': {None: {'x1':1, None:-1}}}
    # leConstraints = {'c2': {None: {'x2':1, 'x3':3, None:1}}}
    # geConstraints = {'c3': {None: {'x1':1, 'x4':1, None:-3}},
    #                  'c4': {None: {'x1':1, 'x2':2, 'x4':1, None:0}}}
    #
    # Note the we have the luxury of dealing only with linear terms.
    var_id_map = {}
    leConstraints = {}
    geConstraints = {}
    eqConstraints = {}
    objectives = {}
    # For each registered component
    for c in self.active_components():

        # Get all subclasses of Constraint
        if issubclass(c, Constraint):
            cons = self.active_components(c)

            # Get the name of the constraint, and the constraint set itself
            for con_set_name in cons:
                con_set = cons[con_set_name]

                # For each indexed constraint in the constraint set
                for ndx in con_set._data:
                    con = con_set._data[ndx]

                    # Process the body
                    terms = self._process_canonical_repn(
                        generate_canonical_repn(con.body, var_id_map))

                    # Process the bounds of the constraint
                    if con._equality:
                        # Equality constraint, only check lower bound
                        lb = self._process_canonical_repn(
                            generate_canonical_repn(con.lower, var_id_map))

                        # Update terms
                        for k in lb:
                            v = lb[k]
                            if k in terms:
                                terms[k] -= v
                            else:
                                terms[k] = -v

                        # Add constraint to equality constraints
                        eqConstraints[(con_set_name, ndx)] = terms
                    else:

                        # Process upper bounds (<= constraints)
                        if con.upper is not None:
                            # Less than or equal to constraint
                            tmp = dict(terms)

                            ub = self._process_canonical_repn(
                                generate_canonical_repn(con.upper, var_id_map))

                            # Update terms
                            for k in ub:
                                if k in terms:
                                    tmp[k] -= ub[k]
                                else:
                                    tmp[k] = -ub[k]

                            # Add constraint to less than or equal to
                            # constraints
                            leConstraints[(con_set_name, ndx)] = tmp

                        # Process lower bounds (>= constraints)
                        if con.lower is not None:
                            # Less than or equal to constraint
                            tmp = dict(terms)

                            lb = self._process_canonical_repn(
                                generate_canonical_repn(con.lower, var_id_map))

                            # Update terms
                            for k in lb:
                                if k in terms:
                                    tmp[k] -= lb[k]
                                else:
                                    tmp[k] = -lb[k]

                            # Add constraint to less than or equal to
                            # constraints
                            geConstraints[(con_set_name, ndx)] = tmp
        elif issubclass(c, Objective):
            # Process objectives
            objs = self.active_components(c)

            # Get the name of the objective, and the objective set itself
            for obj_set_name in objs:
                obj_set = objs[obj_set_name]

                # For each indexed objective in the objective set
                for ndx in obj_set._data:
                    obj = obj_set._data[ndx]
                    # Process the objective
                    terms = self._process_canonical_repn(
                        generate_canonical_repn(obj.expr, var_id_map))

                    objectives[(obj_set_name, ndx)] = terms


    # We now have all the constraints. Add a slack variable for every
    # <= constraint and an excess variable for every >= constraint.
    nSlack = len(leConstraints)
    nExcess = len(geConstraints)

    nConstraints = len(leConstraints) + len(geConstraints) + \
                   len(eqConstraints)
    nVariables = len(colID) + nSlack + nExcess
    nRegVariables = len(colID)

    # Make the arrays
    coefficients = array.array("d", [0]*nConstraints*nVariables)
    constraints = array.array("d", [0]*nConstraints)
    costs = array.array("d", [0]*nVariables)

    # Populate the coefficient matrix
    constraintID = 0

    # Add less than or equal to constraints
    for ndx in leConstraints:
        con = leConstraints[ndx]
        for termKey in con:
            coef = con[termKey]

            if termKey is None:
                # Constraint coefficient
                constraints[constraintID] = -coef
            else:
                # Variable coefficient
                col = colID[termKey]
                coefficients[constraintID*nVariables + col] = coef

        # Add the slack
        coefficients[constraintID*nVariables + nRegVariables + \
                    constraintID] = 1
        constraintID += 1

    # Add greater than or equal to constraints
    for ndx in geConstraints:
        con = geConstraints[ndx]
        for termKey in con:
            coef = con[termKey]

            if termKey is None:
                # Constraint coefficient
                constraints[constraintID] = -coef
            else:
                # Variable coefficient
                col = colID[termKey]
                coefficients[constraintID*nVariables + col] = coef

        # Add the slack
        coefficients[constraintID*nVariables + nRegVariables + \
                    constraintID] = -1
        constraintID += 1

    # Add equality constraints
    for ndx in eqConstraints:
        con = eqConstraints[ndx]
        for termKey in con:
            coef = con[termKey]

            if termKey is None:
                # Constraint coefficient
                constraints[constraintID] = -coef
            else:
                # Variable coefficient
                col = colID[termKey]
                coefficients[constraintID*nVariables + col] = coef

        constraintID += 1

    # Determine cost coefficients
    for obj_name in objectives:
        obj = objectives[obj_name]()
        for var in obj:
            costs[colID[var]] = obj[var]

    # Print the model
    #
    # The goal is to print
    #
    #         var1   var2   var3   ...
    #       +--                     --+
    #       | cost1  cost2  cost3  ...|
    #       +--                     --+
    #       +--                     --+ +-- --+
    # con1  | coef11 coef12 coef13 ...| | eq1 |
    # con2  | coef21 coef22 coef23 ...| | eq2 |
    # con2  | coef31 coef32 coef33 ...| | eq3 |
    #  .    |   .      .      .   .   | |  .  |
    #  .    |   .      .      .    .  | |  .  |
    #  .    |   .      .      .     . | |  .  |

    constraintPadding = 2
    numFmt = "% 1.4f"
    altFmt = "% 1.1g"
    maxColWidth = max(len(numFmt % 0.0), len(altFmt % 0.0))
    maxConstraintColWidth = max(len(numFmt % 0.0), len(altFmt % 0.0))

    # Generate constraint names
    maxConNameLen = 0
    conNames = []
    for name in leConstraints:
        strName = str(name)
        if len(strName) > maxConNameLen:
            maxConNameLen = len(strName)
        conNames.append(strName)
    for name in geConstraints:
        strName = str(name)
        if len(strName) > maxConNameLen:
            maxConNameLen = len(strName)
        conNames.append(strName)
    for name in eqConstraints:
        strName = str(name)
        if len(strName) > maxConNameLen:
            maxConNameLen = len(strName)
        conNames.append(strName)

    # Generate the variable names
    varNames = [None]*len(colID)
    for name in colID:
        tmp_name = " " + name
        if len(tmp_name) > maxColWidth:
            maxColWidth = len(tmp_name)
        varNames[colID[name]] = tmp_name
    for i in xrange(0, nSlack):
        tmp_name = " _slack_%i" % i
        if len(tmp_name) > maxColWidth:
            maxColWidth = len(tmp_name)
        varNames.append(tmp_name)
    for i in xrange(0, nExcess):
        tmp_name = " _excess_%i" % i
        if len(tmp_name) > maxColWidth:
            maxColWidth = len(tmp_name)
        varNames.append(tmp_name)

    # Variable names
    line = " "*maxConNameLen + (" "*constraintPadding) + " "
    for col in xrange(0, nVariables):
        # Format entry
        token = varNames[col]

        # Pad with trailing whitespace
        token += " "*(maxColWidth - len(token))

        # Add to line
        line += " " + token + " "
    print line

    # Cost vector
    print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
          " "*((maxColWidth+2)*nVariables - 4) + "--+"
    line = " "*maxConNameLen + (" "*constraintPadding) + "|"
    for col in xrange(0, nVariables):
        # Format entry
        token = numFmt % costs[col]
        if len(token) > maxColWidth:
            token = altFmt % costs[col]

        # Pad with trailing whitespace
        token += " "*(maxColWidth - len(token))

        # Add to line
        line += " " + token + " "
    line += "|"
    print line
    print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
          " "*((maxColWidth+2)*nVariables - 4) + "--+"

    # Constraints
    print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
          " "*((maxColWidth+2)*nVariables - 4) + "--+" + \
          (" "*constraintPadding) + "+--" + \
          (" "*(maxConstraintColWidth-1)) + "--+"
    for row in xrange(0, nConstraints):
        # Print constraint name
        line = conNames[row] + (" "*constraintPadding) + (" "*(maxConNameLen - len(conNames[row]))) + "|"

        # Print each coefficient
        for col in xrange(0, nVariables):
            # Format entry
            token = numFmt % coefficients[nVariables*row + col]
            if len(token) > maxColWidth:
                token = altFmt % coefficients[nVariables*row + col]

            # Pad with trailing whitespace
            token += " "*(maxColWidth - len(token))

            # Add to line
            line += " " + token + " "

        line += "|" + (" "*constraintPadding) + "|"

        # Add constraint vector
        token = numFmt % constraints[row]
        if len(token) > maxConstraintColWidth:
            token = altFmt % constraints[row]

        # Pad with trailing whitespace
        token += " "*(maxConstraintColWidth - len(token))

        line += " " + token + "  |"
        print line
    print " "*maxConNameLen + (" "*constraintPadding) + "+--" + \
          " "*((maxColWidth+2)*nVariables - 4) + "--+" + \
          (" "*constraintPadding) + "+--" + (" "*(maxConstraintColWidth-1))\
          + "--+"

    return (coefficients, costs, constraints)

def _process_canonical_repn(self, expr):
    """
    Returns a dictionary of {var_name_or_None: coef} values
    """

    terms = {}

    # Get the variables from the canonical representation
    vars = expr.pop(-1, {})

    # Find the linear terms
    linear = expr.pop(1, {})
    for k in linear:
        # FrozeDicts don't support (k, v)-style iteration
        v = linear[k]

        # There's exactly 1 variable in each term
        terms[vars[k.keys()[0]].label] = v

    # Get the constant term, if present
    const = expr.pop(0, {})
    if None in const:
        terms[None] = const[None]

    if len(expr) != 0:
        raise TypeError, "Nonlinear terms in expression"

    return terms




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

import sys
import logging
from pyutilib.misc import LogHandler

from os.path import abspath, dirname, join, normpath
coopr_base = normpath(join(dirname(abspath(__file__)), '..', '..', '..'))

logger = logging.getLogger('coopr.pyomo')
logger.setLevel( logging.WARNING )
logger.addHandler( LogHandler(coopr_base, verbosity=lambda: logger.isEnabledFor(logging.DEBUG)) )



# Nonlinear version of example4.
# Must have a nonlinear solver
# to run this example.
from coopr.pyomo import *
from indexed import model,f

# Reuse the rule from example4 to define the 
# nonlinear constraint
def nonlinear_con_rule(model,i,j,k):
    return model.Z[i,j,k] == f(model,i,j,k,model.X[i,j,k])
model.nonlinear_constraint = Constraint(model.INDEX1,model.INDEX2,rule=nonlinear_con_rule)

# deactivate all constraints representing the piecewise-linear
# version of the model
for i in model.linearized_constraint.index():
    model.linearized_constraint[i].deactivate()
# or just use
#model.linearized_constraint.deactivate()

# initialize the nonlinear variables to 'good' starting points
for idx in model.X.index():
    model.X[idx] = 1.7
    model.Z[idx] = 1.25




from coopr.pyomo import *
from coopr.pyomo.transform import Transformation

__all__ = ["NonlinearTransformation"]

class NonlinearTransformation(Transformation):
    """ Base class for all nonlinear model transformations. """

    def __init__(self, **kwds):
        kwds["name"] = kwds.get("name", "nonlinear_transform")
        super(NonlinearTransformation, self).__init__(**kwds)



# A strictly affine piecewise example
#
#        / (1/3)X - (2/3), -1 <= X <= 2
# Z(X) = | -2X + 4       ,  2 <= X <= 6
#        \ 5X - 38       ,  6 <= X <= 10

from coopr.pyomo import *

# Define the function
# Just like in Pyomo constraint rules, a Pyomo model object
# must be the first argument for the function rule
RANGE_POINTS = {-1.0:-1.0, 2.0:0.0, 6.0:-8.0, 10.0:12.0}
def f(model,x):
    return RANGE_POINTS[x]

model = ConcreteModel()

model.X = Var(bounds=(-1.0,10.0))
model.Z = Var()
model.p = Var(within = NonNegativeReals)
model.n = Var(within = NonNegativeReals)

# See documentation on Piecewise component by typing
# help(Piecewise) in a python terminal after importing coopr.pyomo
# Using BigM constraints with binary variables to represent the piecwise constraints
model.con = Piecewise(model.Z,model.X, # range and domain variables
                      pw_pts=[-1.0,2.0,6.0,10.0],
                      pw_constr_type='EQ',
                      pw_repn='DCC', 
                      f_rule=f)

# minimize the 1-norm distance of Z to 7.0, i.e., |Z-7|
model.pn_con = Constraint(expr= model.Z - 7.0 == model.p - model.n)
model.obj = Objective(rule = lambda model: model.p+model.n , sense=minimize)



from coopr.pyomo.check.checker import IterativeTreeChecker

import ast

class XRange(IterativeTreeChecker):
    def check(self, runner, script, info):
        if isinstance(info, ast.Name):
            if info.id == 'xrange':
                self.problem("'xrange' function was removed in Python 3.")

    def checkerDoc(self):
        return """\
        In Python 3, 'xrange' was removed in favor of 'range', which was
        reimplemented more efficiently. Please change your uses of 'xrange'
        into 'range', e.g.:
            xrange(1,10)       =>       range(1,10)
        """




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['parse_data_commands']

import sys
import re
import os
import os.path
import ply.lex as lex
import ply.yacc as yacc
from pyutilib.misc import flatten_list
from pyutilib.ply import t_newline, t_ignore, _find_column, p_error, ply_init

## -----------------------------------------------------------
##
## Lexer definitions for tokenizing the input
##
## -----------------------------------------------------------

_parse_info = None
debugging = False

reserved = {
    'data' : 'DATA',
    'set' : 'SET',
    'param' : 'PARAM',
    'end' : 'END',
    'import' : 'IMPORT',
    'include' : 'INCLUDE',
    'namespace' : 'NAMESPACE',
}

# Token names
tokens = [
    "COMMA",
    "LBRACE",
    "RBRACE",
#    "NUMBER",
    "SEMICOLON",
    "COLON",
    "COLONEQ",
    "LBRACKET",
    "RBRACKET",
    "LPAREN",
    "RPAREN",
#    "RANGE",
    "WORD",
    "WORDWITHINDEX",
    "WORDWITHSQUOTEDINDEX",
    "STRING",
    "QUOTEDSTRING",
    "FILENAME",
    "EQ",
    "TR",
    "ASTERISK",
    "NONWORD",
] + list(reserved.values())

# Regular expression rules
t_COMMA     = r","
t_LBRACKET  = r"\["
t_RBRACKET  = r"\]"
t_LBRACE  = r"\{"
t_RBRACE  = r"\}"
#t_NUMBER    = r"[0-9]+(\.[0-9]+){0,1}"
t_SEMICOLON = r";"
t_COLON     = r":"
t_COLONEQ   = r":="
t_EQ        = r"="
t_TR        = r"\(tr\)"
#t_LT        = r"<"
#t_GT        = r">"
t_LPAREN    = r"\("
t_RPAREN    = r"\)"
t_ASTERISK  = r"\*"

# Discard comments
def t_COMMENT(t):
    r'\#[^\n]*'
    #global _comment_list
    #_comment_list.append(t.value)

def t_WORDWITHINDEX(t):
    r'[a-zA-Z_0-9][a-zA-Z_0-9\.\-]*\[[a-zA-Z_0-9\.\-,\*]*\]'
    if t.value in reserved:
        t.type = reserved[t.value]    # Check for reserved words
    #t.type = reserved.get(t.value,'WORDWITHINDEX')    # Check for reserved words
    return t

def t_WORDWITHSQUOTEDINDEX(t):
    r'[a-zA-Z_0-9][a-zA-Z_0-9\.\-]*\[[\'\"a-zA-Z_0-9\.\-,\* ]*\]'
    if t.value in reserved:
        t.type = reserved[t.value]    # Check for reserved words
    #t.type = reserved.get(t.value,'WORDWITHSQUOTEDINDEX')    # Check for reserved words
    return t

def t_WORD(t):
    r'[a-zA-Z_0-9][a-zA-Z_0-9\.+\-]*'
    if t.value in reserved:
        t.type = reserved[t.value]    # Check for reserved words
    #t.type = reserved.get(t.value,'WORD')    # Check for reserved words
    return t

def t_STRING(t):
    r'[a-zA-Z_0-9\.+\-]+'
    if t.value in reserved:
        t.type = reserved[t.value]    # Check for reserved words
    #t.type = reserved.get(t.value,'STRING')    # Check for reserved words
    return t

def t_QUOTEDSTRING(t):
    r'"([^"]|\"\")*"|\'([^\']|\'\')*\''
    if t.value in reserved:
        t.type = reserved[t.value]    # Check for reserved words
    #t.type = reserved.get(t.value,'QUOTEDSTRING')    # Check for reserved words
    return t

def t_FILENAME(t):
    r'[a-zA-Z_0-9\./\\]*(/|\\)[a-zA-Z_0-9\-\./\\]*|[a-zA-Z_0-9\./\\]*(/|\\)[a-zA-Z_0-9\-\./\\]*'
    if t.value in reserved:
        t.type = reserved.get(t.value)    # Check for reserved words
    else:
        t.type = 'FILENAME'
    #t.type = reserved.get(t.value,'FILENAME')    # Check for reserved words
    return t

t_NONWORD   = r"[^\.A-Za-z0-9,;:=<>\*\(\)\#{}\[\] \n\t\r]+"

# Error handling rule
def t_error(t):             #pragma:nocover
    raise IOError, "Illegal character '%s'" % t.value[0]
    t.lexer.skip(1)


## -----------------------------------------------------------
##
## Yacc grammar for data commands
##
## -----------------------------------------------------------

def p_expr(p):
    '''expr : statements
            | '''
    if len(p) == 2:
        #print "STMTS",p[1]
        for stmt in p[1]:
            if type(stmt) is list:
                _parse_info[None].append(stmt)
            else:
                for key in stmt:
                    if key in _parse_info:
                        _parse_info[key].append(stmt[key])
                    else:
                        _parse_info[key] = stmt[key]

def p_statements(p):
    '''statements : statements statement
                  | statement
                  | statements NAMESPACE WORD LBRACE statements RBRACE
                  | NAMESPACE WORD LBRACE statements RBRACE '''
    #print "STMT X",p[1:],p[1]
    len_p = len(p)
    if len_p == 3:
        # NB: statements will never be None, but statement *could* be None
        p[0] = p[1]
        if p[2] is not None:
            p[0].append(p[2])
    elif len_p == 2:
        if p[1] is None:
            p[0] = []
        else:
            p[0] = [p[1]]
    elif len_p == 7:
        # NB: statements will never be None
        p[0] = p[1]
        p[0].append({p[3]:p[5]})
    else:
        # NB: statements will never be None
        p[0] = [{p[2] : p[4]}]

def p_statement(p):
    '''statement : SET WORD COLONEQ setdecl SEMICOLON
                 | SET WORD COLONEQ SEMICOLON
                 | SET WORD COLON items COLONEQ setdecl SEMICOLON
                 | SET WORD COLON items COLONEQ SEMICOLON
                 | SET WORDWITHINDEX COLONEQ setdecl SEMICOLON
                 | SET WORDWITHINDEX COLONEQ SEMICOLON
                 | SET WORDWITHSQUOTEDINDEX COLONEQ setdecl SEMICOLON
                 | SET WORDWITHSQUOTEDINDEX COLONEQ SEMICOLON
                 | PARAM items COLONEQ paramdecl SEMICOLON
                 | IMPORT importdecl SEMICOLON
                 | INCLUDE WORD SEMICOLON
                 | INCLUDE QUOTEDSTRING SEMICOLON
                 | DATA SEMICOLON
                 | END SEMICOLON
    '''
    #print "STATEMENT",len(p), p[1:]
    stmt = p[1]
    if stmt == 'set' or stmt == 'param':
        p[0] = flatten_list(p[1:-1])
    elif stmt == 'include':
        p[0] = p[1:-1]
    elif stmt == 'import':
        p[0] = [p[1]]+ p[2]
    else:
        # Not necessary, but nice to document how statement could end up None
        p[0] = None 

def p_setdecl(p):
    '''setdecl : items'''
    p[0] = p[1]

def p_paramdecl(p):
    '''paramdecl : items'''
    p[0] = p[1]

def p_importdecl(p):
    '''importdecl : filename import_options
                  | filename
                  | filename import_options COLON WORD EQ indices variable_options
                  | filename COLON WORD EQ indices variable_options
                  | filename import_options COLON indices variable_options
                  | filename COLON indices variable_options
                  | filename import_options COLON variable_options
                  | filename COLON variable_options
    '''
    tmp = {'filename':p[1]}
    if len(p) == 2:
        p[0] = [tmp, (None,[]), {}]
    elif len(p) == 3:
        tmp.update(p[2])
        p[0] = [tmp, (None,[]), {}]
    elif len(p) == 4:
        p[0] = [tmp, (None,[]), p[3]]
    elif len(p) == 5:
        if p[2] == ':':
            p[0] = [tmp, (None,p[3]), p[4]]
        else:
            tmp.update(p[2])
            p[0] = [tmp, (None,[]), p[4]]
    elif len(p) == 6:
        tmp.update(p[2])
        p[0] = [tmp, (None,p[4]), p[5]]
    elif len(p) == 7:
        p[0] = [tmp, (p[3],p[5]), p[6]]
    elif len(p) == 8:
        tmp.update(p[2])
        p[0] = [tmp, (p[4],p[6]), p[7]]
    else:
        raise IOError, "Unexpected condition"

def p_import_options(p):
    '''import_options : WORD EQ STRING import_options
                      | WORD EQ STRING
                      | WORD EQ QUOTEDSTRING import_options
                      | WORD EQ QUOTEDSTRING
                      | WORD EQ WORD import_options
                      | WORD EQ WORD
                      | WORD EQ PARAM import_options
                      | WORD EQ PARAM
                      | WORD EQ SET import_options
                      | WORD EQ SET
    '''
    tmp = {p[1]:p[3]}
    if len(p) == 4:
        p[0] = tmp
    else:
        tmp.update(p[4])
        p[0] = tmp

def p_variable_options(p):
    '''variable_options : variable variable_options
                        | variable
    '''
    if len(p) == 2:
        p[0] = p[1]
    else:
        p[1].update(p[2])
        p[0] = p[1]

def p_variable(p):
    '''variable : WORD
                | WORD EQ WORD
    '''
    if len(p) == 2:
        p[0] = {p[1]:p[1]}
    else:
        p[0] = {p[3]:p[1]}

def p_indices(p):
    '''indices : LBRACKET WORD index_list RBRACKET
               | LBRACKET WORD RBRACKET
    '''
    if len(p) == 5:
        p[0] = p[3]
        p[0].insert(0,p[2])
    else:
        p[0] = [p[2]]

def p_index_list(p):
    '''index_list : COMMA WORD index_list
                  | COMMA ASTERISK index_list
                  | COMMA WORD
                  | COMMA ASTERISK
    '''
    if len(p) == 4:
        p[0] = p[3]
        p[0].insert(0,p[2])
    else:
        p[0] = [p[2]]

def p_set_template(p):
    '''set_template : LPAREN WORD index_list RPAREN
                | LPAREN ASTERISK index_list RPAREN
                | LPAREN WORD RPAREN
                | LPAREN ASTERISK RPAREN
    '''
    if len(p) == 5:
        p[0] = p[1]+",".join([p[2]]+p[3])+p[4]
    else:
        p[0] = p[1]+p[2]+p[3]

def p_param_template(p):
    '''param_template : LBRACKET WORD index_list RBRACKET
                | LBRACKET ASTERISK index_list RBRACKET
                | LBRACKET WORD RBRACKET
                | LBRACKET ASTERISK RBRACKET
    '''
    if len(p) == 5:
        p[0] = p[1]+",".join([p[2]]+p[3])+p[4]
    else:
        p[0] = p[1]+p[2]+p[3]

# Composite p_item into p_items
# Produces ~3% speedup, but at the cost of readability
def p_items(p):
    '''
    items : items WORD
          | items WORDWITHINDEX
          | items WORDWITHSQUOTEDINDEX
          | items NONWORD
          | items STRING
          | items QUOTEDSTRING
          | items COMMA
          | items COLON
          | items LBRACE
          | items RBRACE
          | items LBRACKET
          | items RBRACKET
          | items TR
          | items LPAREN
          | items RPAREN
          | items ASTERISK
          | items set_template
          | items param_template
          | WORD
          | WORDWITHINDEX
          | WORDWITHSQUOTEDINDEX
          | NONWORD
          | STRING
          | QUOTEDSTRING
          | COMMA
          | COLON
          | LBRACE
          | RBRACE
          | LBRACKET
          | RBRACKET
          | TR
          | LPAREN
          | RPAREN
          | ASTERISK
          | set_template
          | param_template
    '''
    # Locate and handle item as necessary
    single_item = len(p) == 2
    if single_item:
        tmp = p[1]
    else:
        tmp = p[2]
    if tmp[0] == '"' and tmp[-1] == '"' and len(tmp) > 2 and not ' ' in tmp:
        tmp = tmp[1:-1]

    # Grow items list according to parsed item length
    if single_item:
        p[0] = [tmp]
    else:
        # yacc __getitem__ is expensive: use a local list to avoid a
        # getitem call on p[0]
        tmp_lst = p[1]
        tmp_lst.append(tmp)
        p[0] = tmp_lst

def p_filename(p):
    '''filename : WORD
                | STRING
                | QUOTEDSTRING
                | FILENAME
                | WORD COLON FILENAME
    '''
    if len(p) == 2:
        p[0] = p[1]
    else:
        p[0] = p[1]+p[2]+p[3]

#
# the ampl dat file lexer and yaccer only need to be
# created once, so have the corresponding objects
# accessible at module scope.
#

tabmodule = 'parse_table_datacmds'

ampl_dat_lexer = None
ampl_dat_yaccer = None

#
# The function that performs the parsing
#
def parse_data_commands(data=None, filename=None, debug=0, outputdir=None):

    global debugging
    global ampl_dat_lexer
    global ampl_dat_yaccer

    if outputdir is None:
        outputdir = os.path.dirname(os.path.abspath(__file__))+os.sep

    # if the lexer/yaccer haven't been initialized, do so.
    if ampl_dat_lexer is None:
        #
        # Always remove the parser.out file, which is generated to create debugging
        #
        if os.path.exists("parser.out"):        #pragma:nocover
            os.remove("parser.out")
        if debug > 0:                           #pragma:nocover
            #
            # Remove the parsetab.py* files.  These apparently need to be removed
            # to ensure the creation of a parser.out file.
            #
            if os.path.exists("parsetab.py"):
                os.remove("parsetab.py")
            if os.path.exists("parsetab.pyc"):
                os.remove("parsetab.pyc")
            debugging=True

        ampl_dat_lexer = lex.lex()
        #
        tmpsyspath = sys.path
        sys.path.append(outputdir)
        ampl_dat_yaccer = yacc.yacc(debug=debug, tabmodule=tabmodule, outputdir=outputdir)
        sys.path = tmpsyspath

    #
    # Initialize parse object
    #
    global _parse_info
    _parse_info = {}
    _parse_info[None] = []

    #
    # Parse the file
    #
    global _parsedata
    if not data is None:
        _parsedata=data
        ply_init(_parsedata)
        ampl_dat_yaccer.parse(data, lexer=ampl_dat_lexer, debug=debug)
    elif not filename is None:
        f = open(filename, 'r')
        try:
            data = f.read()
        except Exception, e:
            f.close()
            del f
            raise e
        f.close()
        del f
        _parsedata=data
        ply_init(_parsedata)
        ampl_dat_yaccer.parse(data, lexer=ampl_dat_lexer, debug=debug)
    else:
        _parse_info = None
    #
    # Disable parsing I/O
    #
    debugging=False
    return _parse_info



#
# Example 2.6a - Allen Holder
#

from coopr.pyomo import *
from coopr.opt import *

# Instantiate the model
model = AbstractModel()

# Sets and Set Parameters
model.NumSensors = Param(within=NonNegativeIntegers)
model.Sensor = RangeSet(1,model.NumSensors)

# Parameters
model.xPos = Param(model.Sensor, within=NonNegativeIntegers)
model.yPos = Param(model.Sensor, within=NonNegativeIntegers)

# Variables
model.xCentralSensor = Var(within=NonNegativeIntegers)
model.yCentralSensor = Var(within=NonNegativeIntegers)
model.xMax = Var(model.Sensor, within=NonNegativeReals)
model.yMax = Var(model.Sensor, within=NonNegativeReals)

# Objective
def CalcDist(M):
    return sum(M.xMax[s] + M.yMax[s] for s in M.Sensor)
model.Dist = Objective(rule=CalcDist, sense=minimize)

# Constraints

def xEnsureUp(s,M):
    return M.xCentralSensor - M.xPos[s] <= M.xMax[s]
model.xUpBound = Constraint(model.Sensor, rule=xEnsureUp)

def xEnsureLow(s,M):
    return M.xCentralSensor - M.xPos[s] >= -M.xMax[s]
model.xLowBound = Constraint(model.Sensor, rule=xEnsureLow)

def yEnsureUp(s,M):
    return M.yCentralSensor - M.yPos[s] <= M.yMax[s]
model.yUpBound = Constraint(model.Sensor, rule=yEnsureUp)

def yEnsureLow(s,M):
    return M.yCentralSensor - M.yPos[s] >= -M.yMax[s]
model.yLowBound = Constraint(model.Sensor, rule=yEnsureLow)




#  _________________________________________________________________________
#
#  Coopr: A COmmon Optimization Python Repository
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  For more information, see the Coopr README.txt file.
#  _________________________________________________________________________

__all__ = ['Component']

import weakref

from pyutilib.component.core import Plugin, implements
from plugin import IModelComponent


class Component(Plugin):
    """
    This is the base class for all Pyomo modeling components.
    This class is derived from the PyUtilib Plugin class, which
    allows components to be instantiated from a plugin factory.

    Constructor arguments:
        ctype           The class type for the derived subclass
        doc             A text string describing this component
        name            A name for this component

    Public class attributes:
        active          A boolean that is true if this component will be 
                            used to construct a model instance
        doc             A text string describing this component

    Private class attributes:
        _constructed    A boolean that is true if this component has been
                            constructed
        _model          A weakref to the model that owns this component
        _parent         A weakref to the parent block that owns this component
        _type           The class type for the derived subclass
    """

    #
    # This plugin class implements the IModelComponent interface.  This is
    # is not a service plugin, so no plugin instance is created by default.
    #
    implements(IModelComponent, service=False)

    def __init__ (self, **kwds):
        #
        # Get arguments
        #
        self.doc   = kwds.pop('doc', None)
        self.name  = kwds.pop('name', None)
        self._type = kwds.pop('ctype', None)
        if kwds:
            raise ValueError(
                "Unexpected keyword options found while constructing '%s':\n\t%s"
                % ( type(self).__name__, ','.join(sorted(kwds.keys())) ))
        #
        # Verify that ctype has been specified.
        #
        if self._type is None:
            raise DeveloperError, "Must specify a class for the component type!"
        #
        self.active = True
        self._constructed = False
        self._model = None     # Must be a weakref
        self._parent = None    # Must be a weakref

    def __getstate__(self):
        """
        This method must be defined to support pickling because this class
        owns weakrefs for '_model' and '_parent'.
        """
        result = dict(self.__dict__.items())
        if ('_model' in result) and (type(result['_model']) is weakref.ref):
            result['_model'] = result['_model']()
        if ('_parent' in result) and (type(result['_parent']) is weakref.ref):
            result['_parent'] = result['_parent']()
        return result

    def __setstate__(self, state):
        """
        This method must be defined to support pickling because this class
        owns weakrefs for '_model' and '_parent'.
        """
        for (slot_name, value) in state.iteritems():
            self.__dict__[slot_name] = value
        if '_model' in self.__dict__.keys() and self._model is not None and type(self._parent) != weakref.ref:
            self._model = weakref.ref(self._model)
        if '_parent' in self.__dict__.keys() and self._parent is not None and type(self._parent) != weakref.ref:
            self._parent = weakref.ref(self._parent)

    def activate(self):
        """Set the active attribute to True"""
        self.active=True

    def deactivate(self):
        """Set the active attribute to False"""
        self.active=False

    def type(self):
        """Return the class type for this component"""
        return self._type

    def construct(self, data=None):                     #pragma:nocover
        """API definition for constructing components"""
        pass

    def is_constructed(self):                           #pragma:nocover
        """Return True if this class has been constructed"""
        return self._constructed

    def valid_model_component(self):
        """Return True if this can be used as a model component."""
        return True

    def pprint(self, ostream=None, verbose=False):
        """Print component information"""
        if ostream is None:
            ostream = sys.stdout
        print >>ostream, "  ",self.name,":",


class DeveloperError(Exception):
    """
    Exception class used to throw errors that result from
    programming errors, rather than user modeling errors (e.g., a
    component not declaring a 'ctype').
    """

    def __init__(self, val):
        self.parameter = val

    def __str__(self):                                  #pragma:nocover
        return repr(self.parameter)




"""
       / 2x          , 0 <= x <= 1
       | (1/2)x+(3/2), 1 <= x <= 3
f(x) = | -3x+12      , 3 <= x <= 5
       \ 2x-13       , 5 <= x <= 6
"""

from coopr.pyomo import *

DOMAIN_PTS = [float(i) for i in [0,1,3,5,6]]
RANGE_PTS = {0.0:0.0, 1.0:2.0, 3.0:3.0, 5.0:-3.0, 6.0:-1.0}

def F(model,x):
    return RANGE_PTS[x]*model.p

def define_model(**kwds):

    model = ConcreteModel()

    model.x1 = Var(bounds=(0,6)) # domain variable
    model.x2 = Var(bounds=(0,6)) # domain variable
    model.x3 = Var(bounds=(0,6)) # domain variable
    model.x4 = Var(bounds=(0,6)) # domain variable    
    
    model.Fx1 = Var() # range variable
    model.Fx2 = Var() # range variable
    model.Fx3 = Var() # range variable
    model.Fx4 = Var() # range variable
    model.p = Param(initialize=1.0)

    model.obj = Objective(expr=model.Fx1+model.Fx2+model.Fx3+model.Fx4, sense=kwds.pop('sense',maximize))

    model.piecewise1 = Piecewise(model.Fx1,model.x1,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)
    model.piecewise2 = Piecewise(model.Fx2,model.x2,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    model.piecewise3 = Piecewise(model.Fx3,model.x3,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    model.piecewise4 = Piecewise(model.Fx4,model.x4,
                                  pw_pts=DOMAIN_PTS,
                                  f_rule=F, **kwds)

    #Fix the answer for testing purposes
    model.set_answer_constraint1 = Constraint(expr= model.x1 == 0.0)
    model.set_answer_constraint2 = Constraint(expr= model.x2 == 3.0)
    model.set_answer_constraint3 = Constraint(expr= model.x3 == 5.5)
    model.set_answer_constraint4 = Constraint(expr= model.x4 == 6.0)
    
    return model



from coopr.pyomo import *
from coopr.pyomo.transform import Transformation

__all__ = ["AbstractTransformation"]

class AbstractTransformation(Transformation):
    """
    Base class for all model transformations that produce abstract
    models.
    """

    def __init__(self, **kwds):
        kwds["name"] = kwds.get("name", "concrete_transformation")
        super(AbstractTransformation, self).__init__(**kwds)



from coopr.pyomo import *

# Problem borrowed from the lpsolve project documentation
# http://lpsolve.sourceforge.net/5.0/SOS.htm

# Don't forget that you can see that actual input to the solver if you use
# the -k command line flag:

# $ pyomo basic_sos2_example.py --solver=cplex -k

# A standalone SOS2 example, so the data comes from these functions instead of
# a .dat file.  Thus, define all the "cruft" up here, so it's clearer how the
# model is tied together down below.

def c_param_init ( model, v ):
    return (-1, -1, -3, -2, -2)[ v -1 ]  # -1 because Python is 0-based


def b_param_init ( model, c):
    return (30, 30)[ c -1 ]  # -1 because Python is 0-based


def A_param_init ( model, c, v):
    data = (
      (-1, -1,  1,  1,  0),
      ( 1,  0,  1, -3,  0),
    )

    return data[ c -1 ][ v -1 ]  # -1 because Python is 0-based


def obj_rule ( model ):
    objective_expression = sum(
      model.c[ i ] * model.x[ i ]
      for i in model.variable_set
    )

    return objective_expression


def constraint_rule ( model, c):
    constraint_equation = (
      model.b[ c ] >= sum(
        model.A[c, i] * model.x[ i ]
        for i in model.variable_set
      )
    )

    return constraint_equation


x1_constraint_rule = lambda M: M.x[1] <= 40
x2_constraint_rule = lambda M: M.x[2] <= 1
x5_constraint_rule = lambda M: M.x[5] <= 1


###############################################################################
# Cruft above complete, now we actually create and tie the model together

model = AbstractModel()
M = model

M.variable_set   = RangeSet(1, 5)
M.constraint_set = RangeSet(1, 2)

M.c = Param( M.variable_set, rule=c_param_init )

M.A = Param( M.constraint_set, M.variable_set, rule=A_param_init )
M.b = Param( M.constraint_set, rule=b_param_init )

M.x = Var( M.variable_set, within=PositiveReals )

M.obj = Objective( rule=obj_rule, sense=minimize )  # min "c transpose" X

# At first, this is little more than a standard form Ax=b ...
M.constraints   = Constraint( M.constraint_set, rule=constraint_rule )

# ... with a couple of extra constraints ...
M.x1_constraint = Constraint( rule=x1_constraint_rule )
M.x2_constraint = Constraint( rule=x2_constraint_rule )
M.x5_constraint = Constraint( rule=x5_constraint_rule )

# ... and finally, add the constraint for which this example was created
M.x_sos_vars  = SOSConstraint( var=M.x, sos=2 )




from coopr.pyomo import *
import math

model = AbstractModel()

model.N = Param(within=PositiveIntegers)

model.Locations = RangeSet(1,model.N)

model.P = Param(within=RangeSet(1,model.N))

model.M = Param(within=PositiveIntegers)

model.Customers = RangeSet(1,model.M)

def d_rule(model, n, m):
    return math.sin(n*2.33333+m*7.99999)
model.d = Param(model.Locations, model.Customers, rule=d_rule, within=Reals)

model.x = Var(model.Locations, model.Customers, bounds=(0.0,1.0))

model.y = Var(model.Locations, within=Binary)

def rule(model):
    return sum( [model.d[n,m]*model.x[n,m] for n in model.Locations for m in model.Customers] )
model.obj = Objective(rule=rule)

def rule(model, m):
    return (sum( [model.x[n,m] for n in model.Locations]), 1.0)
model.single_x = Constraint(model.Customers, rule=rule)

def rule(model, n, m):
    return (None, model.x[n,m] - model.y[n], 0.0)
model.bound_y = Constraint(model.Locations, model.Customers, rule=rule)

def rule(model):
    return (sum( [model.y[n] for n in model.Locations] ) - model.P, 0.0)
model.num_facilities = Constraint(rule=rule)



#
# Test the canonical expressions
#

import os
import sys
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo import *
from coopr.opt import *
#from coopr.pyomo.base.var import _VarElement
import pyutilib.th as unittest
import pyutilib.services
#from coopr.pyomo.preprocess.simple_preprocessor import SimplePreprocessor
from pyutilib.component.core import PluginGlobals

class frozendict(dict):
    __slots__ = ('_hash',)
    def __hash__(self):
        rval = getattr(self, '_hash', None)
        if rval is None:
            rval = self._hash = hash(frozenset(self.iteritems()))
        return rval


class Test(unittest.TestCase):

    #def setUp(self):
        #
        # Create Model
        #
        #PluginGlobals.pprint()
        #self.plugin = SimplePreprocessor()
        #self.plugin.deactivate_action("compute_canonical_repn")

    def tearDown(self):
        if os.path.exists("unknown.lp"):
            os.unlink("unknown.lp")
        pyutilib.services.TempfileManager.clear_tempfiles()
        #self.plugin.activate_action("compute_canonical_repn")

    def test_abstract_linear_expression(self):
        m = AbstractModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        def obj_rule(model):
            return summation(model.p, model.x)
        m.obj = Objective(rule=obj_rule)
        i = m.create()
        #self.instance.obj[None].expr.pprint()
        rep = generate_canonical_repn(i.obj[None].expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables...
        self.assertEqual(rep[-1], { 0: i.x[1],
                                    1: i.x[2],
                                    2: i.x[3]})
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 2.0, 
                                   1: 4.0, 
                                   2: 6.0 })


    def test_concrete_linear_expression(self):
        m = ConcreteModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        m.obj = Objective(expr=summation(m.p, m.x))
        rep = generate_canonical_repn(m.obj[None].expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables...
        self.assertEqual(rep[-1], { 0: m.x[1],
                                    1: m.x[2],
                                    2: m.x[3]})
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 2.0, 
                                   1: 4.0, 
                                   2: 6.0 })

    def test_linear_expression(self):
        I = range(3)
        x = [Var(bounds=(-1,1)) for i in I]
        expr = sum(2*(i+1)*x[i] for i in I)
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables... 
        self.assertEqual(rep[-1], { 0: x[0],
                                    1: x[1],
                                    2: x[2] })
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 2.0, 
                                   1: 4.0, 
                                   2: 6.0 } )

    def test_complex_linear_expression(self):
        I = range(3)
        x = [Var(bounds=(-1,1)) for i in I]
        expr = 2*x[1] + 3*x[1] + 4*(x[2]+x[1])
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 2 of the 3 variables...
        self.assertEqual(rep[-1], { 0: x[1],
                                    1: x[2] })
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 9.0, 
                                   1: 4.0 } )


    def test_reversed_complex_linear_expression(self):
        I = range(3)
        x = [Var(bounds=(-1,1)) for i in I]
        expr = 4*(x[2]+x[1]) + 2*x[1] + 3*x[1]
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 2 of the 3 variables...
        self.assertEqual(rep[-1], { 1: x[1],
                                    0: x[2] })
        # check the expression encoding
        self.assertEqual(rep[1], { 1: 9.0, 
                                   0: 4.0 } )


    def test_linear_expression_with_constant(self):
        I = range(3)
        x = [Var(bounds=(-1,1)) for i in I]
        expr = 1.2 + 2*x[1] + 3*x[1]
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,0,1]
        self.assertEqual(len(rep), 3)
        self.assertTrue(0 in rep)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 1 of the 3 variables...
        self.assertEqual(rep[-1], { 0: x[1] })
        # check the expression encoding
        self.assertEqual(rep[0], { None: 1.2 } )
        self.assertEqual(rep[1], { 0: 5.0 } )


    def test_complex_linear_expression_with_constant(self):
        I = range(3)
        x = [Var(bounds=(-1,1)) for i in I]
        expr =  2.0*(1.2 + 2*x[1] + 3*x[1]) + 3.0*(1.0+x[1])
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,0,1]
        self.assertEqual(len(rep), 3)
        self.assertTrue(0 in rep)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 1 of the 3 variables...
        self.assertEqual(rep[-1], { 0: x[1] })
        # check the expression encoding
        self.assertEqual(rep[0], { None: 5.4 } )
        self.assertEqual(rep[1], { 0: 13.0 } )


    def test_polynomial_expression(self):
        I = range(4)
        x = [Var(bounds=(-1,1)) for i in I]
        expr = x[1]*(x[1]+x[2]) + x[2]*(x[1]+3.0*x[3]*x[3])
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,2,3]
        self.assertEqual(len(rep), 3)
        self.assertTrue(2 in rep)
        self.assertTrue(3 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 of the 4 variables...
        self.assertEqual(rep[-1], { 0: x[1],
                                    1: x[2],
                                    2: x[3] })
        # check the expression encoding
        self.assertEqual(rep[2], {frozendict({0:2}):1.0, 
                                  frozendict({0:1, 1:1}):2.0})
        self.assertEqual(rep[3], {frozendict({1:1, 2:2}):3.0})


    def test_polynomial_expression_with_fixed(self):
        I = range(4)
        x = [Var() for i in I]
        expr = x[1]*(x[1]+x[2]) + x[2]*(x[1]+3.0*x[3]*x[3])
        x[1].value = 5
        x[1].fixed = True
        rep = generate_canonical_repn(expr)
        print rep
        # rep should only have [-1,0,1,3]
        self.assertEqual(len(rep), 4)
        self.assertTrue(0 in rep)
        self.assertTrue(1 in rep)
        self.assertTrue(3 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 2 of the 4 variables...
        self.assertEqual(rep[-1], { 0: x[2],
                                    1: x[3] })
        # check the expression encoding
        self.assertEqual(rep[0], {None: 25.0})
        self.assertEqual(rep[1], {0: 10.0})
        self.assertEqual(rep[3], {frozendict({0:1, 1:2}):3.0})


    def test_linear_expression_with_constant_division(self):
        m = ConcreteModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        expr = summation(m.p, m.x)/2.0
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables... 
        self.assertEqual(rep[-1], { 0: m.x[1],
                                    1: m.x[2],
                                    2: m.x[3] })
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 1.0, 
                                   1: 2.0, 
                                   2: 3.0 } )

    def test_linear_expression_with_fixed_division(self):
        m = ConcreteModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        m.y = Var(initialize=2.0)
        m.y.fixed = True
        expr = summation(m.p, m.x)/m.y
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables... 
        self.assertEqual(rep[-1], { 0: m.x[1],
                                    1: m.x[2],
                                    2: m.x[3] })
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 1.0, 
                                   1: 2.0, 
                                   2: 3.0 } )

    def test_linear_expression_with_complex_fixed_division(self):
        m = ConcreteModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        m.y = Var(initialize=1.0)
        m.y.fixed = True
        expr = summation(m.p, m.x)/(m.y+1)
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,1]
        self.assertEqual(len(rep), 2)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables... 
        self.assertEqual(rep[-1], { 0: m.x[1],
                                    1: m.x[2],
                                    2: m.x[3] })
        # check the expression encoding
        self.assertEqual(rep[1], { 0: 1.0, 
                                   1: 2.0, 
                                   2: 3.0 } )


    def test_expr_rational_summation(self):
        m = ConcreteModel()
        m.A = RangeSet(1,3)
        def p_init(model, i):
            return 2*i
        m.p = Param(m.A, initialize=p_init)
        m.x = Var(m.A, bounds=(-1,1))
        m.y = Var(initialize=1.0)
        expr = summation(m.p, m.x)/(1+m.y)
        #expr.pprint()
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,None]
        self.assertEqual(len(rep), 2)
        self.assertTrue(None in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 4 variables... 
        self.assertEqual(rep[-1], { 3: m.y,
                                    0: m.x[1],
                                    1: m.x[2],
                                    2: m.x[3] })
        # check the expression encoding
        self.assertIs(rep[None], expr)

    def test_expr_rational(self):
        m = ConcreteModel()
        m.A = RangeSet(1,4)
        m.x = Var(m.A, bounds=(-1,1))
        m.y = Var(initialize=1.0)
        expr = (1.25+m.x[1]+1/m.y) + (2.0+m.x[2])/m.y
        #expr.pprint()
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,None]
        self.assertEqual(len(rep), 2)
        self.assertTrue(None in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 3 variables... 
        self.assertEqual(rep[-1], { 1: m.y,
                                    0: m.x[1],
                                    2: m.x[2] })
        # check the expression encoding
        self.assertIs(rep[None], expr)

    def test_expr_rational_fixed(self):
        m = ConcreteModel()
        m.A = RangeSet(1,4)
        m.x = Var(m.A, bounds=(-1,1))
        m.y = Var(initialize=2.0)
        m.y.fixed = True
        expr = (1.25+m.x[1]+1/m.y) + (2.0+m.x[2])/m.y
        #expr.pprint()
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,0,1]
        self.assertEqual(len(rep), 3)
        self.assertTrue(0 in rep)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have the 2 of 3 variables... 
        self.assertEqual(rep[-1], { 0: m.x[1],
                                    1: m.x[2] })
        # check the expression encoding 
        self.assertEqual(rep[0], {None: 2.75})
        self.assertEqual(rep[1], {0: 1,
                                  1: 0.5 })

    def test_general_nonlinear(self):
        I = range(3)
        x = [Var() for i in I]
        expr = x[1] + 5*cos(x[2])
        #expr.pprint()
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,None]
        self.assertEqual(len(rep), 2)
        self.assertTrue(None in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have 2 variables... 
        self.assertEqual(rep[-1], { 0: x[1],
                                    1: x[2] })
        # check the expression encoding
        self.assertIs(rep[None], expr)
        
    def test_general_nonlinear_fixed(self):
        I = range(3)
        x = [Var() for i in I]
        expr = x[1] + 5*cos(x[2])
        x[2].value = 0
        x[2].fixed = True
        #expr.pprint()
        rep = generate_canonical_repn(expr)
        # rep should only have [-1,0,1]
        self.assertEqual(len(rep), 3)
        self.assertTrue(0 in rep)
        self.assertTrue(1 in rep)
        self.assertTrue(-1 in rep)
        # rep[-1] should have 1 variable... 
        self.assertEqual(rep[-1], { 0: x[1] })
        # check the expression encoding
        self.assertEqual(rep[0], {None: 5.0})
        self.assertEqual(rep[1], {0: 1})

    def test_deterministic_var_labeling(self):
        m = ConcreteModel()
        m.x = Var(initialize=3.0)
        m.y = Var(initialize=2.0)
        exprA = m.x - m.y
        exprB = m.y - m.x

        # Nondeterministic form should not care which expression comes first
        # The nondeterministic form has been deprecated.
        #rep_A = generate_canonical_repn(exprA)
        #rep_B = generate_canonical_repn(exprB)
        #self.assertEqual( rep_A[-1], rep_B[-1] )
        
        # Deterministic form should care which expression comes first
        idMap_1 = {}
        rep_A = generate_canonical_repn(exprA, idMap_1)
        rep_B = generate_canonical_repn(exprB, idMap_1)
        self.assertEqual( rep_A[-1], rep_B[-1] )
        
        idMap_2 = {}
        rep_B = generate_canonical_repn(exprB, idMap_2)
        rep_A = generate_canonical_repn(exprA, idMap_2)
        self.assertEqual( rep_A[-1], rep_B[-1] )

        self.assertNotEqual( idMap_1, idMap_2 )
        

if __name__ == "__main__":
    unittest.main()



import os
import pyutilib.th as unittest
from pyutilib.misc.pyyaml_util import *
import coopr.pyomo.scripting.util as util
import coopr.plugins
from pyutilib.misc import Options
import pickle
import pyutilib
from coopr.pyomo.base import Var

currdir = os.path.dirname(os.path.abspath(__file__))
os.sys.path.append(currdir)

problem_list = ['multidimen_sos2','sos2','multidimen_sos1','sos1','sos1_no_index','sos2_no_index']

def module_available(module):
    try:
        __import__(module)
        return True
    except ImportError:
        return False

def has_python(name):
    if module_available(name):
        return True
    return False

def has_gurobi_lp():
    try:
        gurobi = coopr.plugins.solvers.GUROBI(keepFiles=True)
        available = (not gurobi.executable() is None) and gurobi.available(False)
        return available
    except pyutilib.common.ApplicationError:
        return False

def has_gurobi_nl():
    try:
        gurobi = coopr.plugins.solvers.GUROBI(keepFiles=True)
        available = (not gurobi.executable() is None) and gurobi.available(False)
        asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'gurobi_ampl'})
        return available and (not asl.executable() is None) and asl.available(False)
    except pyutilib.common.ApplicationError:
        return False

def has_cplex_lp():
    try:
        cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
        available = (not cplex.executable() is None) and cplex.available(False)
        return available
    except pyutilib.common.ApplicationError:
        return False
    
def has_cbc_lp():
    try:
        cbc = coopr.plugins.solvers.CBC(keepFiles=True)
        available = (not cbc.executable() is None) and cbc.available(False)
        return available
    except pyutilib.common.ApplicationError:
        return False
    
def has_pico_lp():
    try:
        PICO = coopr.plugins.solvers.PICO(keepFiles=True)
        available = (not PICO.executable() is None) and PICO.available(False)
        return available
    except pyutilib.common.ApplicationError:
        return False

def has_cplex_nl():
    try:
        cplex = coopr.plugins.solvers.CPLEX(keepFiles=True)
        available = (not cplex.executable() is None) and cplex.available(False)
        asl = coopr.plugins.solvers.ASL(keepFiles=True, options={'solver':'cplexamp'})
        return available and (not asl.executable() is None) and asl.available(False)
    except pyutilib.common.ApplicationError:
        return False

def has_glpk_lp():
    try:
        glpk = coopr.plugins.solvers.GLPK(keepFiles=True)
        available = (not glpk.executable() is None) and glpk.available(False)
        return available
    except pyutilib.common.ApplicationError:
        return False

writer_solver = []
if has_python('cplex'):
    writer_solver.append(('python','cplex',True))
else:
    writer_solver.append(('python','cplex',False))

if has_python('gurobipy'):
    writer_solver.append(('python','gurobi',True))
else:
    writer_solver.append(('python','gurobi',False))

if has_cplex_lp():
    writer_solver.append(('lp','cplex',True))
else:
    writer_solver.append(('lp','cplex',False))

if has_gurobi_lp():
    writer_solver.append(('lp','gurobi',True))
else:
    writer_solver.append(('lp','gurobi',False))
    
if has_cplex_nl():
    writer_solver.append(('nl','cplexamp',True))
else:
    writer_solver.append(('nl','cplexamp',False))
    
if has_gurobi_nl():
    writer_solver.append(('nl','gurobi_ampl',True))
else:
    writer_solver.append(('nl','gurobi_ampl',False))
    
# Does not support sos
"""
if has_python('glpk'):
    writer_solver.append(('python','glpk',True))
else:
    writer_solver.append(('python','glpk',False))
"""
# Does not support sos
"""
if has_glpk_lp():
    writer_solver.append(('lp','glpk',True))
else:
    writer_solver.append(('lp','glpk',False))
"""
# Does not support sos 
"""
if has_cbc_lp():
    writer_solver.append(('lp','cbc',True))
else:
    writer_solver.append(('lp','cbc',False))
"""
# Does not support sos since uses glpk
"""
if has_pico_lp():
    writer_solver.append(('lp','pico',True))
else:
    writer_solver.append(('lp','pico',False))
"""

def createTestMethod(pName,problem,solver,writer,do_test):
    
    
    message = solver+"_"+writer+" not available"
    
    @unittest.skipUnless(do_test,message)
    def testMethod(obj):
        # Since we are comparing solutions over different writers to a baseline,
        # we cannot use the .yml solution files as a basis. Therefore we must
        # check that individual variable values are the same. Currently, a 
        # pickled results opbject is used as the baseline solution.

        options = Options()
        options.solver = solver
        options.solver_io = writer
        options.quiet = True
        data = Options(options=options)

        model = obj.m[problem].define_model()
        instance = model.create()

        opt_data = util.apply_optimizer(data, instance=instance)
        
        instance.load(opt_data.results)
        
        baseline_results = obj.b[problem]
        
        for block in instance.all_blocks():
            for variable in block.active_components(Var).itervalues():
                for ndx in variable:
                    var = variable[ndx]
                    bF = baseline_results[var.name]
                    F = var.value
                    if abs(F-bF) > 0.000001:
                        raise IOError, "Difference in baseline solution values and current solution values using:\n" + \
                            "Problem: "+problem+".py\n" + \
                            "Solver: "+solver+"\n" + \
                            "Writer: "+writer+"\n" + \
                            "Variable: "+name+"\n" + \
                            "Solution: "+str(F)+"\n" + \
                            "Baseline: "+str(bF)+"\n"

    return testMethod

def assignTests(cls):
    for PROBLEM in problem_list:
        for writer,solver,do_test in writer_solver:
            attrName = "test_sos_{0}_{1}_{2}".format(PROBLEM,solver,writer)
            setattr(cls,attrName,createTestMethod(attrName,PROBLEM,solver,writer,do_test))

@unittest.skipIf(writer_solver==[], "Can't find a solver.")
class SOSTest(unittest.TestCase):
    
    @classmethod
    def setUpClass(self):
        self.m = {}
        self.b = {}
        for p in problem_list:
            self.m[p] = __import__(p)
            f = open(currdir+'/baselines/'+p+'_baseline_results.pickle','r')
            results = pickle.load(f)
            self.b[p] = results
            f.close()

assignTests(SOSTest)



from coopr.pyomo import *

INDEX_SET = [1,'a']
PIECEWISE_PTS = {1:[1,2,3], 'a':[1,2,3]}
F = lambda x: x**2

def define_model():

    model = ConcreteModel()

    def SOS_indices_init(model,t):
        return [(t,i) for i in xrange(len(PIECEWISE_PTS[t]))]
    def ub_indices_init(model):
        return [(t,i) for t in INDEX_SET for i in xrange(len(PIECEWISE_PTS[t]))]
    
    model.SOS_indices = Set(INDEX_SET,dimen=2, ordered=True, initialize=SOS_indices_init)
    model.ub_indices = Set(ordered=True, dimen=2,initialize=ub_indices_init)

    model.x = Var(INDEX_SET)
    model.Fx = Var(INDEX_SET)
    #Add SOS2 variable to model
    model.y = Var(model.ub_indices,within=NonNegativeReals)

    def constraint1_rule(model,t):
        return model.x[t] == sum(model.y[t,i]*PIECEWISE_PTS[t][i] for i in xrange(len(PIECEWISE_PTS[t])) )
    def constraint2_rule(model,t):
        return model.Fx[t] == sum(model.y[t,i]*F(PIECEWISE_PTS[t][i]) for i in xrange(len(PIECEWISE_PTS[t])) )
    def constraint3_rule(model,t):
        return sum(model.y[t,j] for j in xrange(len(PIECEWISE_PTS[t]))) == 1

    model.obj = Objective(expr=summation(model.Fx), sense=maximize)
    model.constraint1 = Constraint(INDEX_SET,rule=constraint1_rule)
    model.constraint2 = Constraint(INDEX_SET,rule=constraint2_rule)
    model.constraint3 = Constraint(INDEX_SET,rule=constraint3_rule)
    model.SOS_set_constraint = SOSConstraint(INDEX_SET, var=model.y, set=model.SOS_indices, sos=2)

    #Fix the answer for testing purposes
    model.set_answer_constraint1 = Constraint(expr= model.x[1] == 2.5)
    model.set_answer_constraint2 = Constraint(expr= model.x['a'] == 1.5)

    return model

model = define_model()



#
# Unit Tests for Elements of a Model
#
# TestSimpleCon                Class for testing single constraint
# TestArrayCon                Class for testing array of constraint
#

import logging
import os
import sys
import StringIO
from os.path import abspath, dirname
sys.path.insert(0, dirname(dirname(abspath(__file__)))+os.sep+".."+os.sep+"..")
currdir = dirname(abspath(__file__))+os.sep

from coopr.pyomo.base import IntegerSet
from coopr.pyomo import *
from coopr.opt import *
from coopr.pyomo.base.var import _VarElement
import pyutilib.th as unittest
import pyutilib.services

class LogBuffer(object):
    def __init__(self, logger, logLevel=None):
        if isinstance(logger, logging.Logger):
            self.logger = logger
        else:
            self.logger = logging.getLogger(logger)
        self.buffer = StringIO.StringIO()
        if logLevel is None:
            self.old_level = None
        else:
            self.old_level = self.logger.getEffectiveLevel()
            self.logger.setLevel(logLevel)
            
        self.logHandler = logging.StreamHandler(self.buffer)
        self.logger.addHandler(self.logHandler)

    def __del__(self):
        self.close()

    def close(self):
        if self.logger is None:
            return
        if self.old_level is not None:
            self.logger.setLevel(self.old_level)

        self.logger.removeHandler(self.logHandler)
        self.logHandler.flush()
        self.buffer.flush()
        self.logHandler = None
        self.logger = None

    def value(self):
        if self.logHandler is not None:
            self.logHandler.flush();
        self.buffer.flush()
        return self.buffer.getvalue()


class PyomoModel(unittest.TestCase):

    def setUp(self):
        self.model = AbstractModel()

    def construct(self,filename):
        self.instance = self.model.create(filename)


class TestConstraintCreation(unittest.TestCase):
    def setUp(self):
        self.model = AbstractModel()
        self.model.x = Var()
        self.model.y = Var()
        self.model.z = Var()

    def test_tuple_construct_equality(self):
        def rule(model):
            return (0.0, model.x)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         True)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.x)
        self.assertEqual(instance.c.upper,             0)

        def rule(model):
            return (model.x, 0.0)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         True)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.x)
        self.assertEqual(instance.c.upper,             0)

    def test_tuple_construct_inf_equality(self):
        def rule(model):
            return (model.x, float('inf'))
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return (float('inf'), model.x)
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)


    def test_tuple_construct_1sided_inequality(self):
        def rule(model):
            return (None, model.y, 1)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             1)

        def rule(model):
            return (0, model.y, None)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

    def test_tuple_construct_1sided_inf_inequality(self):
        def rule(model):
            return (float('-inf'), model.y, 1)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             1)

        def rule(model):
            return (0, model.y, float('inf'))
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

    def test_tuple_construct_unbounded_inequality(self):
        def rule(model):
            return (None, model.y, None)
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return (float('-inf'), model.y, float('inf'))
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

    def test_tuple_construct_invalid_1sided_inequality(self):
        def rule(model):
            return (model.x, model.y, None)
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return (None, model.y, model.z)
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)


    def test_tuple_construct_2sided_inequality(self):
        def rule(model):
            return (0, model.y, 1)
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             1)

    def test_tuple_construct_invalid_2sided_inequality(self):
        def rule(model):
            return (model.x, model.y, 1)
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return (0, model.y, model.z)
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)



    def test_expr_construct_equality(self):
        def rule(model):
            return 0.0 == model.x
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         True)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.x)
        self.assertEqual(instance.c.upper,             0)

        def rule(model):
            return model.x == 0.0
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         True)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.x)
        self.assertEqual(instance.c.upper,             0)

    def test_expr_construct_inf_equality(self):
        def rule(model):
            return model.x == float('inf')
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return float('inf') == model.x
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)


    def test_expr_construct_1sided_inequality(self):
        def rule(model):
            return model.y <= 1
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             1)

        def rule(model):
            return 0 <= model.y
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             0)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return model.y >= 1
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             1)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return 0 >= model.y
        self.model.c = Constraint(rule=rule)
        instance=self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             0)

    def test_expr_construct_unbounded_inequality(self):
        def rule(model):
            return model.y <= float('inf')
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return float('-inf') <= model.y
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return model.y >= float('-inf')
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

        def rule(model):
            return float('inf') >= model.y
        self.model.c = Constraint(rule=rule)
        instance = self.model.create()
        self.assertEqual(instance.c._equality,         False)
        self.assertEqual(instance.c.lower,             None)
        self.assertIs   (instance.c.body,              instance.y)
        self.assertEqual(instance.c.upper,             None)

    def test_expr_construct_invalid_unbounded_inequality(self):
        def rule(model):
            return model.y <= float('-inf')
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return float('inf') <= model.y
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return model.y >= float('inf')
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)

        def rule(model):
            return float('-inf') >= model.y
        self.model.c = Constraint(rule=rule)
        self.assertRaises(ValueError, self.model.create)


class TestSimpleCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)

    def tearDown(self):
        pass

    def test_set_expr_explicit_multivariate(self):
        """Test expr= option (multivariate expression)"""
        model = ConcreteModel()
        model.x = Var(RangeSet(1,4),initialize=2)
        ans=0
        for i in model.x.keys():
            ans = ans + model.x[i]
        ans = ans >= 0
        ans = ans <= 1
        model.c = Constraint(expr=ans)
        instance = model.create()
        self.assertEqual(instance.c(), 8)
        self.assertEqual(instance.c.body(), 8)
        self.assertEqual(value(instance.c), 8)

    def test_set_expr_explicit_univariate(self):
        """Test expr= option (univariate expression)"""
        model = ConcreteModel()
        model.x = Var(initialize=2)
        ans = model.x >= 0
        ans = ans <= 1
        model.c = Constraint(expr=ans)
        instance = model.create()
        self.assertEqual(instance.c(), 2)
        self.assertEqual(instance.c.body(), 2)
        self.assertEqual(value(instance.c), 2)

    def test_set_expr_undefined_univariate(self):
        """Test expr= option (univariate expression)"""
        model = AbstractModel()
        model.x = Var()
        ans = model.x >= 0
        ans = ans <= 1
        model.c = Constraint(expr=ans)
        instance = model.create()
        self.assertRaises(ValueError, instance.c)
        instance.x = 2
        self.assertEqual(instance.c(), 2)
        self.assertEqual(value(instance.c), 2)

    def test_set_expr_inline(self):
        """Test expr= option (inline expression)"""
        model = ConcreteModel()
        model.A = RangeSet(1,4)
        model.x = Var(model.A,initialize=2)
        model.c = Constraint(expr=0 <= sum(model.x[i] for i in model.A) <= 1)
        instance = model.create()
        self.assertEqual(instance.c(), 8)
        self.assertEqual(value(instance.c), 8)


    def test_rule1(self):
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            ans = ans >= 0
            ans = ans <= 1
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c(), 8)
        self.assertEqual(value(self.instance.c), 8)

    def test_rule2(self):
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            return (0,ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c(), 8)
        self.assertEqual(value(self.instance.c), 8)

    def test_rule3(self):
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            return (0,ans,None)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c(), 8)
        self.assertEqual(value(self.instance.c), 8)

    def test_rule4(self):
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            return (None,ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c(), 8)
        self.assertEqual(value(self.instance.c), 8)

    def test_rule5(self):
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            return (ans,1)
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c(), 8)
        self.assertEqual(value(self.instance.c), 8)

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c.dim(),0)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c.keys(),[])

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),0)


class TestArrayCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2,3,4])

    def tearDown(self):
        pass

    def test_rule_option1(self):
        """Test rule option"""
        def f(model, i):
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(self.instance.c[2](), 16)
        self.assertEqual(len(self.instance.c), 4)

    def test_old_rule_option1(self):
        """Test rule option"""
        buffer = LogBuffer('coopr.pyomo', logging.WARNING)
        def f(model, i):
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(self.instance.c[2](), 16)
        self.assertEqual(len(self.instance.c), 4)
        #self.assertEqual(buffer.value(), """\
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[1])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[2])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[3])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[4])
#""")

    def test_rule_option2(self):
        """Test rule option"""
        def f(model, i):
            if i%2 == 0:
                return Constraint.Skip
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_rule_option3(self):
        """Test rule option"""
        def f(model, i):
            if i%2 == 0:
                return Constraint.Skip
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_rule_option2a(self):
        """Test rule option"""
        @simple_constraint_rule
        def f(model, i):
            if i%2 == 0:
                return None
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_rule_option3a(self):
        """Test rule option"""
        @simple_constraint_rule
        def f(model, i):
            if i%2 == 0:
                return None
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint(self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c.dim(),1)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint(self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c.keys()),0)

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint(self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),0)
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            ans = ans==2
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),1)


class TestConList(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2,3,4])

    def tearDown(self):
        pass

    def test_rule_option1(self):
        """Test rule option"""
        def f(model, i):
            if i > 4:
                return ConstraintList.End
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = ConstraintList(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(self.instance.c[2](), 16)
        self.assertEqual(len(self.instance.c), 4)

    def test_rule_option2(self):
        """Test rule option"""
        def f(model, i):
            if i > 2:
                return ConstraintList.End
            i = 2*i - 1
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = ConstraintList(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_rule_option1a(self):
        """Test rule option"""
        @simple_constraintlist_rule
        def f(model, i):
            if i > 4:
                return None
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = ConstraintList(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(self.instance.c[2](), 16)
        self.assertEqual(len(self.instance.c), 4)

    def test_rule_option2a(self):
        """Test rule option"""
        @simple_constraintlist_rule
        def f(model, i):
            if i > 2:
                return None
            i = 2*i - 1
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = ConstraintList(rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1](), 8)
        self.assertEqual(len(self.instance.c), 2)

    def test_dim(self):
        """Test dim method"""
        self.model.c = ConstraintList(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c.dim(),1)

    def test_keys(self):
        """Test keys method"""
        self.model.c = ConstraintList(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c.keys()),0)

    def test_len(self):
        """Test len method"""
        self.model.c = ConstraintList(noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),0)


class Test2DArrayCon(PyomoModel):

    def setUp(self):
        #
        # Create Model
        #
        PyomoModel.setUp(self)
        self.model.A = Set(initialize=[1,2])

    def tearDown(self):
        pass

    def test_rule_option(self):
        """Test rule option"""
        def f(model, i, j):
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1,1](), 8)
        self.assertEqual(self.instance.c[2,1](), 16)

    def test_old_rule_option(self):
        """Test rule option"""
        buffer = LogBuffer('coopr.pyomo', logging.WARNING)
        def f(model, i, j):
            ans=0
            for j in model.x.keys():
                ans = ans + model.x[j]
            ans *= i
            ans = ans < 0
            ans = ans > 0
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(self.model.A,self.model.A,rule=f)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c[1,1](), 8)
        self.assertEqual(self.instance.c[2,1](), 16)
        #self.assertEqual(buffer.value(), """\
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[1,1])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[1,2])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[2,1])
#DEPRECATION WARNING: Detected pre-3.0 style rule where the model argument appears last (Constraint c[2,2])
#""")

    def test_dim(self):
        """Test dim method"""
        self.model.c = Constraint(self.model.A,self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(self.instance.c.dim(),2)

    def test_keys(self):
        """Test keys method"""
        self.model.c = Constraint(self.model.A,self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c.keys()),0)

    def test_len(self):
        """Test len method"""
        self.model.c = Constraint(self.model.A,self.model.A, noruleinit=True)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),0)
        """Test rule option"""
        def f(model):
            ans=0
            for i in model.x.keys():
                ans = ans + model.x[i]
            ans = ans==2
            return ans
        self.model.x = Var(RangeSet(1,4),initialize=2)
        self.model.c = Constraint(rule=f)
        self.instance = self.model.create()
        self.assertEqual(len(self.instance.c),1)

class MiscConTests(pyutilib.th.TestCase):

    def test_constructor(self):
        a = Constraint(name="b", noruleinit=True)
        self.assertEqual(a.name,"b")
        try:
            a = Constraint(foo="bar", noruleinit=True)
            self.fail("Can't specify an unexpected constructor option")
        except ValueError:
            pass

    def test_contains(self):
        model=AbstractModel()
        model.a=Set(initialize=[1,2,3])
        model.b=Constraint(model.a)
        instance=model.create()
        self.assertEqual(2 in instance.b,False)
        tmp=[]
        for i in instance.b:
            tmp.append(i)
        self.assertEqual(len(tmp),0)

    def test_set_get(self):
        a = Constraint(noruleinit=True)
        #try:
            #a.value = 1
            #self.fail("Can't set value attribute")
        #except AttributeError:
            #pass
        self.assertEqual(a(),None)

    def test_rule(self):
        def rule1(model):
            return Constraint.Skip
        model=AbstractModel()
        model.o = Constraint(rule=rule1)
        instance=model.create()
        try:
            instance=model.create()
        except Exception, e:
            self.fail("Failure to create empty constraint: %s" % str(e))
        #
        def rule1(model):
            return (0.0,model.x,2.0)
        model=AbstractModel()
        model.x = Var(initialize=1.1)
        model.o = Constraint(rule=rule1)
        instance=model.create()
        instance.reset()
        self.assertEqual(instance.o(),1.1)
        #
        def rule1(model, i):
            return Constraint.Skip
        model=AbstractModel()
        model.a=Set(initialize=[1,2,3])
        model.o = Constraint(model.a,rule=rule1)
        try:
            instance=model.create()
        except Exception:
            self.fail("Error generating empty objective")
        #
        def rule1(model):
            return (0.0,1.1,2.0,None)
        model=AbstractModel()
        model.o = Constraint(rule=rule1)
        try:
            instance=model.create()
            self.fail("Can only return tuples of length 2 or 3")
        except ValueError:
            pass

    def test_tuple_constraint_create(self):
        def rule1(model):
            return (0.0,model.x)
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        instance=model.create()
        #
        def rule1(model):
            return (model.y,model.x,model.z)
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #

    def test_expression_constructor_coverage(self):
        def rule1(model):
            expr=model.x
            expr = expr == 0.0
            expr = expr > 1.0
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(TypeError, model.create)
        #
        def rule1(model):
            expr = model.U >= model.x
            expr = expr >= model.L
            return expr
        model=AbstractModel()
        model.x = Var()
        model.L = Param(initialize=0)
        model.U = Param(initialize=1)
        model.o = Constraint(rule=rule1)
        instance=model.create()
        #
        def rule1(model):
            expr = model.x <= model.z
            expr = expr >= model.y
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.x >= model.z
            expr = model.y >= expr
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.y <= model.x
            expr = model.y >= expr
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.x >= model.L
            return expr
        model=AbstractModel()
        model.x = Var()
        model.L = Param(initialize=0)
        model.o = Constraint(rule=rule1)
        instance=model.create()
        #
        def rule1(model):
            expr = model.U >= model.x
            return expr
        model=AbstractModel()
        model.x = Var()
        model.U = Param(initialize=0)
        model.o = Constraint(rule=rule1)
        instance=model.create()

        #
        def rule1(model):
            expr=model.x
            expr = expr == 0.0
            expr = expr < 1.0
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(TypeError, model.create)
        #
        def rule1(model):
            expr = model.U <= model.x
            expr = expr <= model.L
            return expr
        model=AbstractModel()
        model.x = Var()
        model.L = Param(initialize=0)
        model.U = Param(initialize=1)
        model.o = Constraint(rule=rule1)
        instance=model.create()
        #
        def rule1(model):
            expr = model.x >= model.z
            expr = expr <= model.y
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.x <= model.z
            expr = model.y <= expr
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.z = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.y >= model.x
            expr = model.y <= expr
            return expr
        model=AbstractModel()
        model.x = Var()
        model.y = Var()
        model.o = Constraint(rule=rule1)
        self.assertRaises(ValueError, model.create)
        #
        def rule1(model):
            expr = model.x <= model.L
            return expr
        model=AbstractModel()
        model.x = Var()
        model.L = Param(initialize=0)
        model.o = Constraint(rule=rule1)
        instance=model.create()
        #
        def rule1(model):
            expr = model.U <= model.x
            return expr
        model=AbstractModel()
        model.x = Var()
        model.U = Param(initialize=0)
        model.o = Constraint(rule=rule1)
        instance=model.create()

        #
        def rule1(model):
            return model.x+model.x
        model=AbstractModel()
        model.x = Var()
        model.o = Constraint(rule=rule1)
        try:
            instance=model.create()
            self.fail("Cannot return an unbounded expression")
        except ValueError:
            pass
        #

if __name__ == "__main__":
    unittest.main()
